#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtklineresampling.py,v $
## Language:  Python
## Date:      $Date: 2005/09/14 09:49:59 $
## Version:   $Revision: 1.4 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENCE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even 
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
##      PURPOSE.  See the above copyright notices for more information.


import vtk
import sys

import pypes

import vmtklineresampling

vmtkcenterlineresampling = 'vmtkCenterlineResampling'

class vmtkCenterlineResampling(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)
        
        self.Centerlines = None

        self.Length = 0.0;

        self.SetScriptName('vmtkcenterlineresampling')
        self.SetScriptDoc('resample input centerlines with a spline filter')
        self.SetInputMembers([
            ['Centerlines','i','vtkPolyData',1,'','the input centerlines','vmtksurfacereader'],
            ['Length','length','float',1,'(0.0,)','length of the resampling interval']
            ])
        self.SetOutputMembers([
            ['Centerlines','o','vtkPolyData',1,'','the output centerlines','vmtksurfacewriter']
            ])

    def Execute(self):

        if self.Centerlines == None:
            self.PrintError('Error: No input centerlines.')

        lineResampling = vmtklineresampling.vmtkLineResampling()
        lineResampling.Surface = self.Centerlines
        lineResampling.Length = self.Length
        lineResampling.Execute()

        self.Centerlines = lineResampling.Surface


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkimagemipviewer.py,v $
## Language:  Python
## Date:      $Date: 2006/06/20 12:12:05 $
## Version:   $Revision: 1.1 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENCE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even 
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
##      PURPOSE.  See the above copyright notices for more information.

import vtk
import sys

import vtkvmtk
import vmtkrenderer
import pypes

vmtkimagemipviewer = 'vmtkImageMIPViewer'

class vmtkImageMIPViewer(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Image = None
        self.vmtkRenderer = None
        self.OwnRenderer = 0
        self.Display = 1
        self.ArrayName = ''

        self.SampleDistance = 1.0
        self.AutoSampleDistance = 1
        
        self.WindowLevel = [0.0, 0.0]

        self.Volume = None

        self.SetScriptName('vmtkimagemipviewer')
        self.SetScriptDoc('display a 3D image')
        self.SetInputMembers([
            ['Image','i','vtkImageData',1,'','the input image','vmtkimagereader'],
            ['ArrayName','array','str',1,'','name of the array to display'],
            ['vmtkRenderer','renderer','vmtkRenderer',1,'','external renderer'],
            ['SampleDistance','sampledistance','float',1,'(0.0,)','the distance at sample projections are generated'],
            ['AutoSampleDistance','autosampledistance','bool',1,'','toggle automatic sample distance'],
            ['WindowLevel','windowlevel','float',2,'','the window/level for generating the rendering'],
            ['Display','display','bool',1,'','toggle rendering']
            ])
        self.SetOutputMembers([
            ['Image','o','vtkImageData',1,'','the output image','vmtkimagewriter']
            ])

    def BuildView(self):

        if not self.vmtkRenderer:
            self.vmtkRenderer = vmtkrenderer.vmtkRenderer()
            self.vmtkRenderer.Initialize()
            self.OwnRenderer = 1

        self.vmtkRenderer.RegisterScript(self) 

        if self.Volume:
            self.vmtkRenderer.Renderer.RemoveVolume(self.Volume)

        if (self.ArrayName != ''):
            self.Image.GetPointData().SetActiveScalars(self.ArrayName)

        scalarRange = [0.0, 0.0]

        if self.WindowLevel[0] > 0.0:
            scalarRange = [self.WindowLevel[1] - self.WindowLevel[0]/2.0, self.WindowLevel[1] + self.WindowLevel[0]/2.0]
        else:
            scalarRange = self.Image.GetScalarRange()

        colorTransferFunction = vtk.vtkColorTransferFunction()
        colorTransferFunction.AddRGBPoint(scalarRange[0], 0.0, 0.0, 0.0)
        colorTransferFunction.AddRGBPoint(scalarRange[1], 1.0, 1.0, 1.0)

        volumeMapper = vtk.vtkFixedPointVolumeRayCastMapper()
        volumeMapper.SetInput(self.Image)
        volumeMapper.SetBlendModeToMaximumIntensity()
        if self.AutoSampleDistance:
            volumeMapper.AutoAdjustSampleDistancesOn()
        else:
            volumeMapper.SetSampleDistance(self.SampleDistance)
        
        volumeProperty = vtk.vtkVolumeProperty()
        volumeProperty.ShadeOn()
        volumeProperty.SetInterpolationTypeToLinear()
        volumeProperty.SetColor(colorTransferFunction)
        
        self.Volume = vtk.vtkVolume()
        self.Volume.SetMapper(volumeMapper)
        self.Volume.SetProperty(volumeProperty)
      
        self.vmtkRenderer.Renderer.AddVolume(self.Volume)

        if (self.Display == 1):
            self.vmtkRenderer.Render()

        if self.OwnRenderer:
            self.vmtkRenderer.Deallocate()

    def Execute(self):

        if (self.Image == None) & (self.Display == 1):
            self.PrintError('Error: no Image.')
 
        self.BuildView()
       
if __name__=='__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



__all__ = [
    'vmtkboundarylayer2',
    'vmtkdijkstradistancetopoints',
    'vmtkdistancetospheres',
    'vmtkgeodesicsurfaceresolution',
    'vmtkmeshaddexternallayer',
    'vmtkmeshclipcenterlines',
    'vmtkmeshtetrahedralize2',
    'vmtkmeshviewer2',
    'vmtkmeshwriter2',
    'vmtksurfaceresolution',
    'vmtksurfacewriter2',
    'vmtksurfaceextractinnercylinder',
    'vmtkthreshold',
    'vmtkmeshmerge',
    'vmtkentityrenumber',
  ]

for item in __all__:
        exec('from '+item+' import *')




#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkbranchextractor.py,v $
## Language:  Python
## Date:      $Date: 2006/02/23 09:31:39 $
## Version:   $Revision: 1.11 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENCE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even 
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
##      PURPOSE.  See the above copyright notices for more information.


import vtk
import vtkvmtk
import sys

import pypes

vmtkbranchextractor = 'vmtkBranchExtractor'

class vmtkBranchExtractor(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)
        
        self.Centerlines = None

        self.RadiusArrayName = ''
	
        self.BlankingArrayName = 'Blanking'
        self.GroupIdsArrayName = 'GroupIds'
        self.CenterlineIdsArrayName = 'CenterlineIds'
        self.TractIdsArrayName = 'TractIds'

        self.SetScriptName('vmtkbranchextractor')
        self.SetInputMembers([
            ['Centerlines','i','vtkPolyData',1,'','','vmtksurfacereader'],
            ['RadiusArrayName','radiusarray','str',1],
            ['GroupIdsArrayName','groupidsarray','str',1],
            ['CenterlineIdsArrayName','centerlineidsarray','str',1],
            ['TractIdsArrayName','tractidsarray','str',1],
				    ['BlankingArrayName','blankingarray','str',1],
            ])
        self.SetOutputMembers([
            ['Centerlines','o','vtkPolyData',1,'','','vmtksurfacewriter'],
            ['GroupIdsArrayName','groupidsarray','str',1],
            ['CenterlineIdsArrayName','centerlineidsarray','str',1],
            ['TractIdsArrayName','tractidsarray','str',1],
      	    ['BlankingArrayName','blankingarray','str',1]
            ])

    def Execute(self):

        if not self.Centerlines:
            self.PrintError('Error: No input centerlines.')

        branchExtractor = vtkvmtk.vtkvmtkCenterlineBranchExtractor()
        branchExtractor.SetInput(self.Centerlines)
        branchExtractor.SetBlankingArrayName(self.BlankingArrayName)
        branchExtractor.SetRadiusArrayName(self.RadiusArrayName)
        branchExtractor.SetGroupIdsArrayName(self.GroupIdsArrayName)
        branchExtractor.SetCenterlineIdsArrayName(self.CenterlineIdsArrayName)
        branchExtractor.SetTractIdsArrayName(self.TractIdsArrayName)
        branchExtractor.Update()

        self.Centerlines = branchExtractor.GetOutput()

        if self.Centerlines.GetSource():
            self.Centerlines.GetSource().UnRegisterAllOutputs()

if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkdistancetospheres.py,v $
## Language:  Python
## Date:      $$
## Version:   $$

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENCE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even 
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
##      PURPOSE.  See the above copyright notices for more information.

## Note: this class was contributed by 
##       Tangui Morvan
##       Kalkulo AS
##       Simula Research Laboratory

## This class computes the euclidean from a set of user-selected spheres to a surface

import vtk
import sys

import vtkvmtk
import vtkvmtkcontrib
import vmtkrenderer
import pypes

vmtkdistancetospheres = 'vmtkDistanceToSpheres'


class vmtkDistanceToSpheres(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)
        
        self.Surface = None
        self.DistanceToSpheresArrayName = 'DistanceToSpheres'
        self.DistanceOffset = 0.
        self.DistanceScale = 1.
        self.MinDistance = 0.
        self.MaxDistance = -1.
        self.Spheres = vtk.vtkPolyData()
        self.vmtkRenderer = None
        self.OwnRenderer = 0
        self.DisplayArray = False
        self.SurfaceMapper = None
        self.CurrentSphereId = -1        
        self.SphereWidget = None
        self.Opacity = 1.
        self.SpheresActor = None
        self.ScalarBarActor = None
	self.InteractionMode = 0
	self.ExamineSurface = None
	self.ExamineSpheres = vtk.vtkPolyData()
	self.ExamineSpheresActor = None
	self.ExamineText = None
        
        self.SetScriptName('vmtkdistancetospheres')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','the input surface','vmtksurfacereader'],
            ['DistanceToSpheresArrayName','distancetospheresarray','str',1,'','array storing the distances'],
            ['DistanceOffset','offset','float',1,'','offset added to the distances'],
            ['DistanceScale','scale','float',1.,'','scale applied to the distances'],
            ['MinDistance','mindistance','float',1,'','minimum value for the distances'],
            ['MaxDistance','maxdistance','float',1,'','maximum value for the distances'],
            ['Opacity','opacity','float',1,'(0.0,1.0)','object opacities in the scene'],
            ['vmtkRenderer','renderer','vmtkRenderer',1,'','external renderer']
            ])
        self.SetOutputMembers([
            ['Surface','o','vtkPolyData',1,'','','vmtksurfacewriter']
            ])
    
    def DistanceParametersValidator(self,text):
        if not text:
            return 1
        splitText = text.strip().split(' ')
        if len(splitText) not in range(1,5):
            return 0
        try:
            for i in range(1,len(splitText)+1):
                float(splitText[i-1])
        except ValueError:
            return 0
        return 1
    
    
    def ComputeDistances(self):
        distanceToSpheresFilter = vtkvmtkcontrib.vtkvmtkPolyDataDistanceToSpheres()
        distanceToSpheresFilter.SetInput(self.Surface)
        distanceToSpheresFilter.SetSpheres(self.Spheres)
        distanceToSpheresFilter.SetDistanceOffset(self.DistanceOffset)
        distanceToSpheresFilter.SetDistanceScale(self.DistanceScale)
        distanceToSpheresFilter.SetMinDistance(self.MinDistance);
        distanceToSpheresFilter.SetMaxDistance(self.MaxDistance);
        distanceToSpheresFilter.SetDistanceToSpheresArrayName(self.DistanceToSpheresArrayName)
        distanceToSpheresFilter.Update()
        return distanceToSpheresFilter.GetOutput()
      
    
    def InitializeSpheres(self):
	if (self.InteractionMode==0):
	    self.Spheres.Initialize()
	    seedPoints = vtk.vtkPoints()
	    self.Spheres.SetPoints(seedPoints)
	    self.Spheres.GetPointData().Initialize()
	    seedRadii = vtk.vtkDoubleArray()
	    self.Spheres.GetPointData().SetScalars(seedRadii)
	    self.CurrentSphereId = -1
	    self.SphereWidget.Off()
	else:
	    self.ExamineSpheres.Initialize()
	    spherePoints = vtk.vtkPoints()
	    self.ExamineSpheres.SetPoints(spherePoints)
	    self.ExamineSpheres.GetPointData().Initialize()
	    sphereRadii = vtk.vtkDoubleArray()
	    self.ExamineSpheres.GetPointData().SetScalars(sphereRadii)
        
    
    def PlaceSphere(self):
        if self.CurrentSphereId == -1:
            return
        self.SphereWidget.SetCenter(self.Spheres.GetPoint(self.CurrentSphereId))
        self.SphereWidget.SetRadius(self.Spheres.GetPointData().GetScalars().GetValue(self.CurrentSphereId))


    def SphereCallback(self,widget,event_string):
        if self.CurrentSphereId == -1:
            return
        minRadius = self.Surface.GetLength()*0.001
        if self.SphereWidget.GetRadius() < minRadius:
            self.SphereWidget.SetRadius(minRadius)
        self.Spheres.GetPoints().SetPoint(self.CurrentSphereId,self.SphereWidget.GetCenter())
        self.Spheres.GetPointData().GetScalars().SetValue(self.CurrentSphereId,self.SphereWidget.GetRadius())
        self.Spheres.Modified()
    
    def UndoCallback(self,obj):
	self.InitializeSpheres()
        self.Spheres.Modified()
        self.vmtkRenderer.RenderWindow.Render()

    def PickCallback(self,obj):
	    picker = vtk.vtkCellPicker()
	    picker.SetTolerance(1E-4 * self.Surface.GetLength())
	    eventPosition = self.vmtkRenderer.RenderWindowInteractor.GetEventPosition()
	    #eventPosition = obj.GetEventPosition()
	    result = picker.Pick(float(eventPosition[0]),float(eventPosition[1]),0.0,self.vmtkRenderer.Renderer)
	    if result == 0:
		return
	    pickPosition = picker.GetPickPosition()
            if (self.InteractionMode==0):	    
		self.CurrentSphereId = self.Spheres.GetPoints().InsertNextPoint(pickPosition)
		self.Spheres.GetPointData().GetScalars().InsertNextValue(self.Surface.GetLength()*0.01)
		self.Spheres.Modified()
		self.PlaceSphere()
		self.SphereWidget.On()
	    else:
		pickedCellPointIds = self.Surface.GetCell(picker.GetCellId()).GetPointIds()
		minDistance = 1E10
		pickedPointId = -1
		for i in range(pickedCellPointIds.GetNumberOfIds()):
		    distance = vtk.vtkMath.Distance2BetweenPoints(pickPosition,self.Surface.GetPoint(pickedCellPointIds.GetId(i)))
		    if distance < minDistance:
			minDistance = distance
			pickedPointId = pickedCellPointIds.GetId(i)
		if pickedPointId == -1:
		    pickedPointId = pickedCellPointIds.GetId(0)
		point = self.Surface.GetPoint(pickedPointId)
		self.ExamineSpheres.GetPoints().InsertNextPoint(point)
		length = 0.
		array = self.ExamineSurface.GetPointData().GetArray(self.DistanceToSpheresArrayName)
		if (array):
		    length = array.GetComponent(pickedPointId,0)
		self.ExamineSpheres.GetPointData().GetScalars().InsertNextValue(length)
		self.ExamineSpheres.Modified()
            self.vmtkRenderer.RenderWindow.Render()


    def IncreaseSphereRadiusCallback(self,obj):
	if self.CurrentSphereId != -1:
	    #increase sphere radius
            newval = self.Spheres.GetPointData().GetScalars().GetValue(self.CurrentSphereId) + self.Surface.GetLength()*0.01
            self.Spheres.GetPointData().GetScalars().SetValue(self.CurrentSphereId,newval)
            self.Spheres.Modified()
            self.PlaceSphere()
            self.vmtkRenderer.RenderWindow.Render()

    def DecreaseSphereRadiusCallback(self,obj):
	if self.CurrentSphereId != -1:
            #decrease sphere radius
            newval = self.Spheres.GetPointData().GetScalars().GetValue(self.CurrentSphereId) - self.Surface.GetLength()*0.01
            if newval> 0:
                self.Spheres.GetPointData().GetScalars().SetValue(self.CurrentSphereId,newval)
                self.Spheres.Modified()
                self.PlaceSphere()
                self.vmtkRenderer.RenderWindow.Render()

    def NextCallback(self,obj):
	if self.CurrentSphereId != -1:
            self.CurrentSphereId = (self.CurrentSphereId + 1) % self.Spheres.GetNumberOfPoints();
            self.PlaceSphere()
            self.vmtkRenderer.RenderWindow.Render()

    def PreviousCallback(self,obj):
	if self.CurrentSphereId != -1:
            self.CurrentSphereId = (self.CurrentSphereId - 1) % self.Spheres.GetNumberOfPoints();
            self.PlaceSphere()
            self.vmtkRenderer.RenderWindow.Render()

    def DistancesCallback(self,obj):
	    self.DisplayArray = not self.DisplayArray
            if self.DisplayArray:
                self.ExamineSurface = self.ComputeDistances()
                self.SurfaceMapper.SetInput(self.ExamineSurface)
                self.ExamineSurface.GetPointData().SetActiveScalars(self.DistanceToSpheresArrayName)
                array = self.ExamineSurface.GetPointData().GetScalars()
                if (array):
                    array.Modified()
                    self.SurfaceMapper.SetScalarRange(array.GetRange(0))
                    self.ScalarBarActor.VisibilityOn()
            else:
                self.SurfaceMapper.SetInput(self.Surface)
                self.ScalarBarActor.VisibilityOff()
            self.SurfaceMapper.SetScalarVisibility(self.DisplayArray)
            self.vmtkRenderer.RenderWindow.Render()

    def ParametersCallback(self,obj):
	    queryString = 'Please input new parameters :\nDistanceOffset('+str(self.DistanceOffset)+') [DistanceScale('+str(self.DistanceScale)+') MinDistance('+str(self.MinDistance)+') MaxDistance('+str(self.MaxDistance)+'): '
            inputString = self.InputText(queryString,self.DistanceParametersValidator)
            splitInputString = inputString.strip().split(' ')
            if len(splitInputString) >= 1 and splitInputString[0] != '':
                self.DistanceOffset = float(splitInputString[0])
            if len(splitInputString) >= 2:
                self.DistanceScale = float(splitInputString[1])
            if len(splitInputString) >= 3:
                self.MinDistance = float(splitInputString[2])
            if len(splitInputString) >= 4:
                self.MaxDistance = float(splitInputString[3])

    def SwitchModeCallback(self,obj):
	    #Switch beetween examien and interact mode
	    if self.InteractionMode == 0:
		self.InteractionMode = 1
		self.ExamineSurface = self.ComputeDistances()
		self.SpheresActor.VisibilityOff()
		self.SphereWidget.Off()
		self.ExamineSpheresActor.VisibilityOn()
		self.ExamineText.VisibilityOn()
		self.InitializeSpheres()
	    else:
		self.InteractionMode = 0
		#Compute the distances
		self.SpheresActor.VisibilityOn()
		self.ExamineSpheresActor.VisibilityOff()
		self.ExamineText.VisibilityOff()
		if (self.CurrentSphereId!=-1):
		    self.SphereWidget.On()
	    self.vmtkRenderer.RenderWindow.Render()

    def Execute(self):

        if self.Surface == None:
            self.PrintError('Error: No input surface.')

        if not self.vmtkRenderer:
          self.vmtkRenderer = vmtkrenderer.vmtkRenderer()
          self.vmtkRenderer.Initialize()
          self.OwnRenderer = 1
          self.vmtkRenderer.RegisterScript(self)  
        glyphs = vtk.vtkGlyph3D()
        glyphSource = vtk.vtkSphereSource()
        glyphSource.SetRadius(1)
        glyphs.SetInput(self.Spheres)
        glyphs.SetSource(glyphSource.GetOutput())
        glyphs.SetScaleModeToScaleByScalar()
        glyphs.SetScaleFactor(1.)
        glyphMapper = vtk.vtkPolyDataMapper()
        glyphMapper.SetInput(glyphs.GetOutput())
        glyphMapper.ScalarVisibilityOff()
        self.SpheresActor = vtk.vtkActor()
        self.SpheresActor.SetMapper(glyphMapper)
        self.SpheresActor.GetProperty().SetColor(1.0,0.0,0.0)
        self.SpheresActor.GetProperty().SetOpacity(self.Opacity)
        self.SpheresActor.PickableOff()
        self.vmtkRenderer.Renderer.AddActor(self.SpheresActor)
        
	examineGlyphs = vtk.vtkGlyph3D()
        examineGlyphSource = vtk.vtkSphereSource()
        examineGlyphSource.SetRadius(1)
        examineGlyphs.SetInput(self.ExamineSpheres)
        examineGlyphs.SetSource(examineGlyphSource.GetOutput())
        examineGlyphs.SetScaleModeToScaleByScalar()
        examineGlyphs.SetScaleFactor(1.)
        examineGlyphMapper = vtk.vtkPolyDataMapper()
        examineGlyphMapper.SetInput(examineGlyphs.GetOutput())
        examineGlyphMapper.ScalarVisibilityOff()
        self.ExamineSpheresActor = vtk.vtkActor()
        self.ExamineSpheresActor.SetMapper(examineGlyphMapper)
        self.ExamineSpheresActor.GetProperty().SetColor(0.0,1.0,0.0)
        self.ExamineSpheresActor.GetProperty().SetOpacity(self.Opacity)
        self.ExamineSpheresActor.PickableOff()
	self.ExamineSpheresActor.VisibilityOff()
        self.vmtkRenderer.Renderer.AddActor(self.ExamineSpheresActor)
	
	self.vmtkRenderer.AddKeyBinding('u','Undo.',self.UndoCallback)
	self.vmtkRenderer.AddKeyBinding('space','Pick.',self.PickCallback)
	self.vmtkRenderer.AddKeyBinding('+','Increase sphere radius.',self.IncreaseSphereRadiusCallback)
	self.vmtkRenderer.AddKeyBinding('-','Decrease sphere radius.',self.DecreaseSphereRadiusCallback)
	self.vmtkRenderer.AddKeyBinding('n','Skip to next sphere.',self.NextCallback)
	self.vmtkRenderer.AddKeyBinding('v','Skip to previous sphere.',self.PreviousCallback)
	self.vmtkRenderer.AddKeyBinding('d','Show distances graph.',self.DistancesCallback)
	self.vmtkRenderer.AddKeyBinding('a','Change parameters.',self.ParametersCallback)
	self.vmtkRenderer.AddKeyBinding('w','Switch modes.',self.SwitchModeCallback)
        
	#self.vmtkRenderer.RenderWindowInteractor.AddObserver("KeyPressEvent", self.KeyPressed)
        
        self.SurfaceMapper = vtk.vtkPolyDataMapper()
        self.SurfaceMapper.SetInput(self.Surface)
        self.SurfaceMapper.SetScalarVisibility(self.DisplayArray)
        surfaceActor = vtk.vtkActor()
        surfaceActor.SetMapper(self.SurfaceMapper)
        surfaceActor.GetProperty().SetOpacity(self.Opacity)
        self.vmtkRenderer.Renderer.AddActor(surfaceActor)
        
        self.ScalarBarActor = vtk.vtkScalarBarActor()
        self.ScalarBarActor.SetLookupTable(self.SurfaceMapper.GetLookupTable())
        self.ScalarBarActor.GetLabelTextProperty().ItalicOff()
        self.ScalarBarActor.GetLabelTextProperty().BoldOff()
        self.ScalarBarActor.GetLabelTextProperty().ShadowOff()
        self.ScalarBarActor.SetLabelFormat('%.2f')
        self.ScalarBarActor.SetTitle('distances')
        self.ScalarBarActor.VisibilityOff()
        self.vmtkRenderer.Renderer.AddActor(self.ScalarBarActor)
        
        
        self.SphereWidget = vtk.vtkSphereWidget()
        self.SphereWidget.SetInteractor(self.vmtkRenderer.RenderWindowInteractor)
        self.SphereWidget.AddObserver("InteractionEvent", self.SphereCallback)
        
	self.ExamineText = vtk.vtkTextActor()
	self.ExamineText.SetInput("Examine Mode")
	self.ExamineText.GetPositionCoordinate().SetCoordinateSystemToNormalizedViewport()
	self.ExamineText.SetPosition(0.05,0.95)
	self.ExamineText.VisibilityOff()
	self.vmtkRenderer.Renderer.AddActor2D(self.ExamineText)
        
        self.InputInfo('Please position the mouse and press space to add spheres, \'u\' to undo\n')
        
        
        any = 0
        while any == 0:
            self.InitializeSpheres()
            self.vmtkRenderer.Render()
            any = self.Spheres.GetNumberOfPoints()
        
        self.Surface = self.ComputeDistances()

        if self.Surface.GetSource():
            self.Surface.GetSource().UnRegisterAllOutputs()

        if self.OwnRenderer:
            self.vmtkRenderer.Deallocate()

if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkmeshclipper.py,v $
## Language:  Python
## Date:      $Date: 2006/05/26 12:35:13 $
## Version:   $Revision: 1.8 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENCE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even 
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
##      PURPOSE.  See the above copyright notices for more information.

import vtk
import sys

import vmtkrenderer
import pypes

vmtkmeshclipper = 'vmtkMeshClipper'

class vmtkMeshClipper(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Mesh = None
        self.vmtkRenderer = None
        self.OwnRenderer = 0

        self.Actor = None
        self.BoxWidget = None
        self.Planes = None

        self.SetScriptName('vmtkmeshclipper')
        self.SetScriptDoc('interactively clip a mesh with a box')
        self.SetInputMembers([
            ['Mesh','i','vtkUnstructuredGrid',1,'','the input mesh','vmtkmeshreader'],
            ['vmtkRenderer','renderer','vmtkRenderer',1,'','external renderer']
            ])
        self.SetOutputMembers([
            ['Mesh','o','vtkUnstructuredGrid',1,'','the output mesh','vmtkmeshwriter']
            ])

    def InteractCallback(self):
        if self.BoxWidget.GetEnabled() == 1:
            self.BoxWidget.SetEnabled(0)
        else:
            self.BoxWidget.SetEnabled(1)

    def ClipCallback(self, obj):
        if self.BoxWidget.GetEnabled() != 1:
            return
        self.BoxWidget.GetPlanes(self.Planes)
        self.Clipper.Update()
        self.Mesh.DeepCopy(self.Clipper.GetClippedOutput())
        mapper = vtk.vtkDataSetMapper()
        mapper.SetInput(self.Mesh)
        mapper.ScalarVisibilityOff()
        self.Actor.SetMapper(mapper)
        self.BoxWidget.Off()

    def Display(self):

      	self.BoxWidget.SetInput(self.Mesh)
      	self.BoxWidget.PlaceWidget()

        self.vmtkRenderer.Render()

    def Execute(self):

        if (self.Mesh == None):
            self.PrintError('Error: no Mesh.')

        self.Planes = vtk.vtkPlanes()
        self.Clipper = vtk.vtkClipDataSet()
        self.Clipper.SetInput(self.Mesh)
        self.Clipper.SetClipFunction(self.Planes)
        self.Clipper.GenerateClippedOutputOn()
        self.Clipper.InsideOutOn()
        
        if not self.vmtkRenderer:
            self.vmtkRenderer = vmtkrenderer.vmtkRenderer()
            self.vmtkRenderer.Initialize()
            self.OwnRenderer = 1

        self.vmtkRenderer.RegisterScript(self) 

        mapper = vtk.vtkDataSetMapper()
        mapper.SetInput(self.Mesh)
        mapper.ScalarVisibilityOff()
        self.Actor = vtk.vtkActor()
        self.Actor.SetMapper(mapper)
        self.vmtkRenderer.Renderer.AddActor(self.Actor)

        self.BoxWidget = vtk.vtkBoxWidget()
        self.BoxWidget.SetInteractor(self.vmtkRenderer.RenderWindowInteractor)
        self.BoxWidget.GetFaceProperty().SetColor(0.6,0.6,0.2)
        self.BoxWidget.GetFaceProperty().SetOpacity(0.25)

        self.vmtkRenderer.AddKeyBinding('i','Interact.', self.InteractCallback)
        self.vmtkRenderer.AddKeyBinding('space','Clip.', self.ClipCallback)

        self.Display()

        if self.OwnRenderer:
            self.vmtkRenderer.Deallocate()
        
        if self.Mesh.GetSource():
            self.Mesh.GetSource().UnRegisterAllOutputs()


if __name__=='__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtksurfacetriangle.py,v $
## Language:  Python
## Date:      $Date: 2006/07/17 09:53:14 $
## Version:   $Revision: 1.8 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENCE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even 
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
##      PURPOSE.  See the above copyright notices for more information.


import sys
import vtk

import pypes

vmtksurfacetriangle = 'vmtkSurfaceTriangle'

class vmtkSurfaceTriangle(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)
        
        self.Surface = None

        self.SetScriptName('vmtksurfacetriangle')
        self.SetScriptDoc('convert all cells in a surface to linear triangles.')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','the input surface','vmtksurfacereader']
            ])
        self.SetOutputMembers([
            ['Surface','o','vtkPolyData',1,'','the output surface','vmtksurfacewriter']
            ])

    def Execute(self):

        if self.Surface == None:
            self.PrintError('Error: No input surface.')

        cleaner = vtk.vtkCleanPolyData()
        cleaner.SetInput(self.Surface)
        cleaner.Update()

        triangleFilter = vtk.vtkTriangleFilter()
        triangleFilter.SetInput(cleaner.GetOutput())
        triangleFilter.Update()

        self.Surface = triangleFilter.GetOutput()

        if self.Surface.GetSource():
            self.Surface.GetSource().UnRegisterAllOutputs()


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()




#!/usr/bin/env python

## Program:   PypeS
## Module:    $RCSfile: pype.py,v $
## Language:  Python
## Date:      $Date: 2006/07/07 10:45:42 $
## Version:   $Revision: 1.18 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENCE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even 
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
##      PURPOSE.  See the above copyright notices for more information.


import sys
import os.path 

pype = 'Pype'

class NullOutputStream(object):
  
    def __init__(self):
        pass

    def write(self,text):
        pass

    def flush(self):
        pass


class Pype(object):

    def __init__(self):
        self.ScriptObjectList = []
        self.ScriptList = []
        self.AutoPipe = 1
        self.LogOn = 1
        self.ScriptName = 'pype'
        self.ExitOnError = 1
        self.InputStream = sys.stdin
        self.OutputStream = sys.stdout
        self.Arguments = None

    def GetUsageString(self):
        usageString = 'Usage: pype --nolog --noauto --query firstScriptName -scriptOptionName scriptOptionValue --pipe secondScriptName -scriptOptionName scriptOptionValue -scriptOptionName @firstScriptName.scriptOptionName -id 2 --pipe thirdScriptName -scriptOptionName @secondScriptName-2.scriptOptionName'
        return usageString

    def SetOutputStreamToNull(self):
        self.OutputStream = NullOutputStream()
        
    def PrintLog(self,logMessage,indent=0):
        if not self.LogOn:
            return
        indentUnit = '    '
        indentation = ''
        for i in range(indent):
            indentation = indentation + indentUnit
        self.OutputStream.write(indentation + logMessage + '\n')
        
    def PrintError(self,logMessage):
        self.OutputStream.write(logMessage + '\n')

    def SetArgumentsString(self,argumentsString):
        if '"' not in argumentsString:
            self.Arguments = argumentsString.split()
        import re
        quoteRe = re.compile('(\".*?\")')
        splitArguments = quoteRe.split(argumentsString)
        arguments = []
        for splitArgument in splitArguments:
            arg = splitArgument
            if splitArgument.startswith('"') and splitArgument.endswith('"'):
                arg = splitArgument[1:-1]
                arguments.append(arg)
            else:
                arguments.extend(arg.strip().split())
            if '"' in arg:
                self.PrintError('Error: non-matching quote found')
        self.Arguments = arguments

    def ParseArguments(self):
        self.ScriptList = []
        arguments = self.Arguments[:]
        if os.path.basename(arguments[0]).strip() in ['pyperun']:
            arguments.remove(arguments[0])
            pypeArguments = []
            for arg in arguments:
                if arg[:2] == '--':
                    pypeArguments.append(arg)
                else:
                    break
            if '--help' in pypeArguments:
                self.PrintLog(self.GetUsageString())
                return
            if '--noauto' in pypeArguments:
                self.AutoPipe = 0
            if '--nolog' in pypeArguments:
                self.LogOn = 0
            if '--query' in pypeArguments:
                queryScripts = arguments[arguments.index('--query')+1:]
                for queryScript in queryScripts:
                    exec('import '+queryScript)
                    exec('allScripts = '+queryScript+'.__all__')
                    self.PrintLog('\n'.join(allScripts))
                return
            for arg in pypeArguments:
                arguments.remove(arg)
        if 'FILE' in arguments:
            text = ''
            self.OutputStream.write('\nThe current pype contains filename placeholders identified by the string FILE.')
            self.OutputStream.write('\nEvery occcurence of FILE is meant to be replaced with an actual file path.\n')
            self.OutputStream.write('\nPress \'c\' to continue and be authomatically prompted for the required filenames.')
            self.OutputStream.write('\nPress \'e\' to exit. You can still edit your pype manually and run it back.\n\n')
            while text not in ['c','e']:
                self.OutputStream.write('> ')
                text = self.InputStream.readline().rstrip('\n')
            if text == 'e':
                return
            elif text =='c':
                while 'FILE' in arguments:
                    arguments[arguments.index('FILE')] = 'BROWSER'
        while '--pipe' in arguments:
            scriptSlice = arguments[:arguments.index('--pipe')]
            self.ScriptList.append([os.path.splitext(os.path.split(scriptSlice[0])[1])[0],scriptSlice[1:]])
            arguments = arguments[arguments.index('--pipe')+1:]
        scriptSlice = arguments[:]
        if not arguments:
            return
        self.ScriptList.append([os.path.splitext(os.path.split(scriptSlice[0])[1])[0],scriptSlice[1:]])
           
    def GetCompatibleMember(self,member,script):
        pushedInputMembers = [scriptMember for scriptMember in script.InputMembers if scriptMember.Pushed]
        compatibleOutputMembers = [scriptMember for scriptMember in pushedInputMembers + script.OutputMembers if scriptMember.AutoPipe and (scriptMember.MemberName == member.MemberName) and (scriptMember.MemberType == member.MemberType)]
        if not compatibleOutputMembers:
            return None
        return compatibleOutputMembers[0]
    
    def AutoPipeScriptObject(self,scriptObject):
        self.PrintLog('Automatic piping ' + scriptObject.ScriptName)
        for memberEntry in scriptObject.InputMembers:
            if not memberEntry.AutoPipe:
                continue
            if memberEntry.MemberType == 'id':
                continue
            if memberEntry.MemberType == 'handle':
                continue
           
            candidateScriptObjectList = [candidateScriptObject for candidateScriptObject in self.ScriptObjectList if self.GetCompatibleMember(memberEntry,candidateScriptObject)]
            if not candidateScriptObjectList:
                continue
            pipedScriptObject = candidateScriptObjectList[-1]
            pipedMember = self.GetCompatibleMember(memberEntry,pipedScriptObject)
            memberEntry.MemberPipe = pipedScriptObject.ScriptName + '-' + str(pipedScriptObject.Id) + '.' + pipedMember.MemberName
            self.PrintLog(memberEntry.MemberName + ' = ' + memberEntry.MemberPipe,1)

    def GetScriptObject(self,scriptName,scriptId):
        for scriptObject in self.ScriptObjectList:
            if (scriptObject.ScriptName == scriptName) & (scriptObject.Id == scriptId):
                return scriptObject

    def ExplicitPipeScriptObject(self,scriptObject):
        self.PrintLog('Explicit piping ' + scriptObject.ScriptName)
        for memberEntry in scriptObject.InputMembers:
            memberName  = memberEntry.MemberName
            option = memberEntry.OptionName
            memberType  = memberEntry.MemberType
            if memberEntry.ExplicitPipe == 'None':
                memberEntry.MemberPipe = None
                exec ('scriptObject.'+memberEntry.MemberName+'= None')
                continue
            if memberEntry.ExplicitPipe:
                pipedArgument = memberEntry.ExplicitPipe
                splitPipedArgument = pipedArgument.split('.')
                if (len(splitPipedArgument)<2):
                    self.PrintError('Error: invalid option piping: '+pipedArgument)
                upstreamPipedModuleName = splitPipedArgument[0]
                if not upstreamPipedModuleName:
                    upstreamPipedModuleName = self.ScriptObjectList[-1].ScriptName
                upstreamPipedOption = ''
                if (len(splitPipedArgument) == 2):
                    upstreamPipedOption = splitPipedArgument[1]
                else:
                    self.PrintError('Error: invalid option piping: '+pipedArgument)

                upstreamPipedId = ''
                splitUpstreamPipedModuleName = upstreamPipedModuleName.split('-')
                if  (len(splitUpstreamPipedModuleName) > 1):
                    upstreamPipedModuleName = splitUpstreamPipedModuleName[-2]
                    upstreamPipedId = splitUpstreamPipedModuleName[-1]
                    
                if not upstreamPipedOption:
                    self.PrintError('Error: invalid option piping: '+pipedArgument)

                candidateScriptObjectList = []

                if upstreamPipedModuleName:
                    candidateScriptObjectList = [candidateScriptObject for candidateScriptObject in self.ScriptObjectList if candidateScriptObject.ScriptName == upstreamPipedModuleName]
                    if upstreamPipedId:
                        candidateScriptObjectList = [candidateScriptObject for candidateScriptObject in candidateScriptObjectList if upstreamPipedId == candidateScriptObject.Id]
                     
                if not candidateScriptObjectList:
                    self.PrintError('Error: invalid option piping: '+pipedArgument)
                    continue
                    
                pipedScriptObject = candidateScriptObjectList[-1]
                 
                candidatePipedMembers = [member for member in pipedScriptObject.OutputMembers + pipedScriptObject.InputMembers if upstreamPipedOption == member.OptionName]

                if not candidatePipedMembers:
                    self.PrintError('Error: invalid option piping: '+pipedArgument)
                    continue

                pipedMember = candidatePipedMembers[0]
               
                memberEntry.MemberPipe = pipedScriptObject.ScriptName + '-' + str(pipedScriptObject.Id) + '.' + pipedMember.MemberName
                self.PrintLog(memberName+' = '+memberEntry.MemberPipe,1)
               
    def PipeScriptObject(self,scriptObject):
        for memberEntry in [member for member in scriptObject.InputMembers if member.MemberPipe and not member.MemberValue]:
            pipedScriptName = memberEntry.MemberPipe.split('.')[0].split('-')[0]
            pipedScriptId = memberEntry.MemberPipe.split('.')[0].split('-')[1]
            pipedMemberName = memberEntry.MemberPipe.split('.')[1]
            previousScriptObjects = self.ScriptObjectList[:]
            if scriptObject in previousScriptObjects:
                previousScriptObjects = previousScriptObjects[:previousScriptObjects.index(scriptObject)]
            candidatePipedScriptObjects = [candidateScriptObject for candidateScriptObject in previousScriptObjects if (candidateScriptObject.ScriptName == pipedScriptName) and (candidateScriptObject.Id == pipedScriptId)]
            pipedScriptObject = candidatePipedScriptObjects[-1]
            exec ('scriptObject.'+memberEntry.MemberName+'='+'pipedScriptObject.'+pipedMemberName)
               
    def Execute(self):
        self.ScriptObjectList = []
        for scriptNameAndArguments in self.ScriptList:
            self.PrintLog('')
            scriptName = scriptNameAndArguments[0]
            moduleName = scriptName
            scriptArguments = scriptNameAndArguments[1]
            imported = True
            try:
                exec('import '+ moduleName)
            except ImportError, e:
                self.PrintError('No module named ' + moduleName + '\n' + e)
                break
            scriptObjectClassName = ''
            exec ('scriptObjectClassName =  '+moduleName+'.'+moduleName)
            moduleScriptObjectClassName = moduleName+'.'+scriptObjectClassName
            self.PrintLog('Creating ' + scriptObjectClassName + ' instance.')
            scriptObject = 0
            exec ('scriptObject = '+moduleScriptObjectClassName+'()')
            scriptArguments = scriptNameAndArguments[1]
            scriptObject.Arguments = scriptArguments
            scriptObject.LogOn = self.LogOn
            if self.InputStream:
                scriptObject.InputStream = self.InputStream
            if self.OutputStream:
                scriptObject.OutputStream = self.OutputStream
            scriptObject.ExitOnError = self.ExitOnError
            if self.AutoPipe:
                self.AutoPipeScriptObject(scriptObject)
            self.PrintLog('Parsing options ' + scriptObject.ScriptName)
            execute = scriptObject.ParseArguments()
            if not execute:
                return
            if scriptObject.Disabled:
                self.PrintLog('\n' + scriptObject.ScriptName + ' is disabled. Bypassing it.')
                continue
            self.ExplicitPipeScriptObject(scriptObject)
            self.PipeScriptObject(scriptObject)
            scriptObject.PrintInputMembers()
            scriptObject.IORead()
            self.PrintLog('Executing ' + scriptObject.ScriptName + ' ...')
            scriptObject.Execute()
            self.PrintLog('Done executing ' + scriptObject.ScriptName + '.')
            scriptObject.IOWrite()
            scriptObject.PrintOutputMembers()
            self.ScriptObjectList.append(scriptObject)

        for scriptObject in self.ScriptObjectList:
            scriptObject.Deallocate()

def PypeRun(arguments):
  
    pipe = Pype()
    pipe.ExitOnError = 0
    pipe.SetArgumentsString(arguments)
    pipe.ParseArguments()
    pipe.Execute()
    return pipe
  

if __name__=='__main__':
    from vmtk import pypes
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()

#    pipe = Pype()
#    pipe.Arguments = sys.argv
#    pipe.ParseArguments()
#    pipe.Execute()




#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkdistancetocenterlines.py,v $
## Language:  Python
## Date:      $Date: 2005/09/14 09:48:31 $
## Version:   $Revision: 1.5 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENCE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even 
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
##      PURPOSE.  See the above copyright notices for more information.


import vtk
import vtkvmtk
import sys

import pypes

vmtkdistancetocenterlines = 'vmtkDistanceToCenterlines'

class vmtkDistanceToCenterlines(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)
        
        self.Surface = None
        self.Centerlines = None
        self.UseRadiusInformation = 0
        self.EvaluateTubeFunction = 0
        self.EvaluateCenterlineRadius = 0
        self.ProjectPointArrays = 0
        self.DistanceToCenterlinesArrayName = 'DistanceToCenterlines'
        self.RadiusArrayName = ''
        
        self.SetScriptName('vmtkdistancetocenterlines')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','','vmtksurfacereader'],
            ['Centerlines','centerlines','vtkPolyData',1,'','','vmtksurfacereader'],
            ['UseRadiusInformation','useradius','bool',1],
            ['EvaluateTubeFunction','tubefunction','bool',1],
            ['EvaluateCenterlineRadius','centerlineradius','bool',1],
            ['ProjectPointArrays','projectarrays','bool',1],
            ['DistanceToCenterlinesArrayName','distancetocenterlinesarray','str',1],
            ['RadiusArrayName','radiusarray','str',1]
            ])
        self.SetOutputMembers([
            ['Surface','o','vtkPolyData',1,'','','vmtksurfacewriter']
            ])

    def Execute(self):

        if self.Surface == None:
            self.PrintError('Error: No input surface.')

        if self.Centerlines == None:
            self.PrintError('Error: No input centerlines.')

        distanceToCenterlinesFilter = vtkvmtk.vtkvmtkPolyDataDistanceToCenterlines()
        distanceToCenterlinesFilter.SetInput(self.Surface)
        distanceToCenterlinesFilter.SetCenterlines(self.Centerlines)
        distanceToCenterlinesFilter.SetUseRadiusInformation(self.UseRadiusInformation)
        distanceToCenterlinesFilter.SetEvaluateTubeFunction(self.EvaluateTubeFunction)
        distanceToCenterlinesFilter.SetEvaluateCenterlineRadius(self.EvaluateCenterlineRadius)
        distanceToCenterlinesFilter.SetProjectPointArrays(self.ProjectPointArrays)
        distanceToCenterlinesFilter.SetDistanceToCenterlinesArrayName(self.DistanceToCenterlinesArrayName)
        distanceToCenterlinesFilter.SetCenterlineRadiusArrayName(self.RadiusArrayName)
        distanceToCenterlinesFilter.Update()

        self.Surface = distanceToCenterlinesFilter.GetOutput()

        if self.Surface.GetSource():
            self.Surface.GetSource().UnRegisterAllOutputs()


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

import sys
import math
import numpy
import vtk
from vmtk import pypes
from vmtk import vmtkscripts
from vmtk import vtkvmtk

vmtkmeshmerge = 'vmtkMeshMerge'

class vmtkMeshMerge(pypes.pypeScript):
    def __init__(self):
        pypes.pypeScript.__init__(self)

        self.SetScriptName("vmtkmeshmerge")
        self.SetScriptDoc('Merge two or three meshes into one.')

        self.Mesh1 = None
        self.Mesh2 = None
        self.Mesh3 = None
        self.Mesh = None
        self.CellEntityIdsArrayName = "CellEntityIds"
        self.CellEntityIdOffset1 = 0
        self.CellEntityIdOffset2 = 0
        self.CellEntityIdOffset3 = 0

        # Member info: name, cmdlinename, typename, num, default, desc[, defaultpipetoscript]
        self.SetInputMembers([
                ['Mesh1', 'mesh1', 'vtkUnstructuredGrid', 1, '',
                 'the first of meshes to merge', 'vmtkmeshreader'],
                ['Mesh2', 'mesh2', 'vtkUnstructuredGrid', 1, '',
                 'the second of meshes to merge', 'vmtkmeshreader'],
                ['Mesh3', 'mesh3', 'vtkUnstructuredGrid', 1, '',
                 '(optional) the third of meshes to merge', 'vmtkmeshreader'],
                ['CellEntityIdsArrayName', 'entityidsarray', 'str', 1, '',
                 'name of the array where entity ids have been stored'],
                ['CellEntityIdOffset1', 'cellentityidoffset1', 'int', 1, '',
                 'offset added to cell entity ids from mesh1', ''],
                ['CellEntityIdOffset2', 'cellentityidoffset2', 'int', 1, '',
                 'offset added to cell entity ids from mesh2', ''],
                ['CellEntityIdOffset3', 'cellentityidoffset3', 'int', 1, '',
                 'offset added to cell entity ids from mesh3', ''],
                ])
        self.SetOutputMembers([
                ['Mesh', 'o', 'vtkUnstructuredGrid', 1, '',
                 'the output mesh', 'vmtkmeshwriter'],
                ['CellEntityIdsArrayName', 'entityidsarray', 'str', 1, '',
                 'name of the array where entity ids have been stored'],
                ])

    def Execute(self):
        if self.Mesh1 == None:
            self.PrintError('Error: No Mesh1.')
        if self.Mesh2 == None:
            self.PrintError('Error: No Mesh2.')

        def addIds(mesh, offset):
            if offset != 0:
                cellids = mesh.GetCellData().GetScalars(self.CellEntityIdsArrayName)
                for i in range(cellids.GetNumberOfTuples()):
                    cellids.SetValue(i, cellids.GetValue(i) + offset)

        addIds(self.Mesh1, self.CellEntityIdOffset1)
        addIds(self.Mesh2, self.CellEntityIdOffset2)
        if self.Mesh3 != None:
            addIds(self.Mesh3, self.CellEntityIdOffset3)

        merger = vtkvmtk.vtkvmtkAppendFilter()
        merger.AddInput(self.Mesh1)
        merger.AddInput(self.Mesh2)
        if self.Mesh3 != None:
            merger.AddInput(self.Mesh3)
        merger.SetMergeDuplicatePoints(1)
        merger.Update()

        self.Mesh = merger.GetOutput()

if __name__ == '__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkimagesmoothing.py,v $
## Language:  Python
## Date:      $Date: 2006/07/17 09:53:14 $
## Version:   $Revision: 1.8 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENCE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even 
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
##      PURPOSE.  See the above copyright notices for more information.


import vtk
import sys

import pypes

vmtkimagesmoothing = 'vmtkImageSmoothing'

class vmtkImageSmoothing(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)
        
        self.Image = None

        self.StandardDeviation = 1.0
        self.RadiusFactor = 5.0
        self.Dimensionality = 3

        self.SetScriptName('vmtkimagesmoothing')
        self.SetScriptDoc('smooth an image with a Gaussian kernel')
        self.SetInputMembers([
            ['Image','i','vtkImageData',1,'','the input image','vmtkimagereader'],
            ['StandardDeviation','sigma','float',1,'(0.0,)','the standard deviation of the Gaussian in real space units'],
            ['RadiusFactor','radiusfactor','float',1,'(0.0,)','the factor specifing the width of the discrete Gaussian kernel'],
            ['Dimensionality','dimensionality','int',1,'(2,3)','the dimensionality of the convolution']
            ])
        self.SetOutputMembers([
            ['Image','o','vtkImageData',1,'','the output image','vmtkimagewriter']
            ])

    def Execute(self):

        if self.Image == None:
            self.PrintError('Error: No input image.')

        spacing = self.Image.GetSpacing()
        pixelStandardDeviations = [self.StandardDeviation/spacing[0],self.StandardDeviation/spacing[1],self.StandardDeviation/spacing[2]]

        smoothingFilter = vtk.vtkImageGaussianSmooth()
        smoothingFilter.SetInput(self.Image)
        smoothingFilter.SetStandardDeviations(pixelStandardDeviations)
        smoothingFilter.SetRadiusFactor(self.RadiusFactor)
        smoothingFilter.SetDimensionality(self.Dimensionality)
        smoothingFilter.Update()

        self.Image = smoothingFilter.GetOutput()


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkrenderer.py,v $
## Language:  Python
## Date:      $Date: 2006/05/26 12:35:13 $
## Version:   $Revision: 1.8 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENCE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even 
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
##      PURPOSE.  See the above copyright notices for more information.

import vtk
import sys
import os

import pypes
import vtkvmtk

vmtkrenderer = 'vmtkRenderer'


class vmtkRendererInputStream(object):

    def __init__(self,renderer):
        self.renderer = renderer

    def readline(self):
        self.renderer.EnterTextInputMode()
        return self.renderer.CurrentTextInput

    def prompt(self,text):
        self.renderer.TextInputQuery = text
        self.renderer.CurrentTextInput = None
        self.renderer.UpdateTextInput()


class vmtkRenderer(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.vmtkRenderer = self

        self.Renderer = None
        self.RenderWindow = None
        self.RenderWindowInteractor = None

        self.Camera = None

        self.WindowSize = [800, 600]
        self.Background = [0.1, 0.1, 0.2]

        self.PointSmoothing = 1
        self.LineSmoothing = 1
        self.PolygonSmoothing = 0

        self.TextInputMode = 0
        self.ExitAfterTextInputMode = True
        self.ExitTextInputCallback = None

        self.TextInputActor = None
        self.TextInputQuery = None

        self.CurrentTextInput = None
        self.InputPosition = [0.25, 0.1]

        self.TextActor = None
        self.Position = [0.001, 0.05]

        self.KeyBindings = {}
 
        self.ScreenshotMagnification = 4

        self.UseRendererInputStream = True

        self.SetScriptName('vmtkrenderer')
        self.SetScriptDoc('renderer used to make several viewers use the same rendering window')
        self.SetInputMembers([
            ['WindowSize','size','int',2,'','size of the rendering window'],
            ['PointSmoothing','pointsmoothing','bool',1,'','toggle rendering smooth points'],
            ['LineSmoothing','linesmoothing','bool',1,'','toggle rendering smooth lines'],
            ['PolygonSmoothing','polygonsmoothing','bool',1,'','toggle rendering smooth polygons'],
            ['Background','background','float',3,'','background color of the rendering window'],
            ['ScreenshotMagnification','magnification','int',1,'','magnification to apply to the rendering window when taking a screenshot']])
        self.SetOutputMembers([
            ['vmtkRenderer','o','vmtkRenderer',1,'','the renderer']])

    def ResetCameraCallback(self,obj):
	self.Renderer.ResetCamera()

    def ScreenshotCallback(self, obj):
        filePrefix = 'vmtk-screenshot'
        fileNumber = 0
        fileName = "%s-%d.png" % (filePrefix,fileNumber)
        homeDir = os.getenv("HOME")
        existingFiles = os.listdir(homeDir)
        while fileName in existingFiles:
            fileNumber += 1
            fileName = "%s-%d.png" % (filePrefix,fileNumber)
        self.PrintLog('Saving screenshot to ' + fileName)
        windowToImage = vtk.vtkWindowToImageFilter()
        windowToImage.SetInput(self.RenderWindow)
        windowToImage.SetMagnification(self.ScreenshotMagnification)
        windowToImage.Update()
        self.RenderWindow.Render()
        writer = vtk.vtkPNGWriter()
        writer.SetInput(windowToImage.GetOutput())
        writer.SetFileName(os.path.join(homeDir,fileName))
        writer.Write()

    def QuitRendererCallback(self, obj):
        self.PrintLog('Quit renderer')
        self.Renderer.RemoveActor(self.TextActor)
        self.RenderWindowInteractor.ExitCallback()

    def ResetCameraCallback(self, obj):
        self.Renderer.ResetCamera()
        self.RenderWindow.Render()

    def UpdateTextInput(self):
        if self.TextInputQuery:
            if self.CurrentTextInput or self.CurrentTextInput == '':
                self.TextInputActor.SetInput(self.TextInputQuery+self.CurrentTextInput+'_')
            else:
                self.TextInputActor.SetInput(self.TextInputQuery)
            self.Renderer.AddActor(self.TextInputActor)
        else:
            self.Renderer.RemoveActor(self.TextInputActor)
        self.RenderWindow.Render()

    def KeyPressCallback(self, obj, event):
        return

    def CharCallback(self, obj, event):
        key = self.RenderWindowInteractor.GetKeySym()
        if self.TextInputMode:
            if key in ['Return','Enter']:
                self.ExitTextInputMode()
                return
            if key.startswith('KP_'):
                key = key[3:]
            if key == 'space':
                key = ' '
            elif key in ['minus','Subtract']:
                key = '-'
            elif key in ['period','Decimal']:
                key = '.'
            elif len(key) > 1 and key not in ['Backspace','BackSpace']:
                key = None
            if key in ['Backspace','BackSpace']:
                textInput = self.CurrentTextInput
                if len(textInput) > 0:
                    self.CurrentTextInput = textInput[:-1]
            elif key:
                self.CurrentTextInput += key
            self.UpdateTextInput()
            return

        if key in self.KeyBindings and self.KeyBindings[key]['callback'] != None:
            self.KeyBindings[key]['callback'](obj)
        else:
            if key == 'plus':
                key = '+'
            if key == 'minus':
                key = '-'
            if key == 'equal':
                key = '='
            if key in self.KeyBindings and self.KeyBindings[key]['callback'] != None:
                self.KeyBindings[key]['callback'](obj)

    def AddKeyBinding(self, key, text, callback=None, group='1'):
        self.KeyBindings[key] = {'text': text, 'callback': callback, 'group': group}

    def RemoveKeyBinding(self, key):
        if key in self.KeyBindings:   
            del self.KeyBindings[key]

    def PromptAsync(self, queryText, callback):
        self.TextInputQuery = queryText
        self.CurrentTextInput = None
        self.ExitTextInputCallback = callback
        self.UpdateTextInput()
        self.EnterTextInputMode(interactive=0)

    def EnterTextInputMode(self,interactive=1):
        self.CurrentTextInput = ''
        self.Renderer.AddActor(self.TextInputActor)
        self.Renderer.RemoveActor(self.TextActor)
        self.UpdateTextInput()
        self.TextInputMode = 1
        self.Render(interactive)
    
    def ExitTextInputMode(self):
        self.Renderer.RemoveActor(self.TextInputActor)
        self.Renderer.AddActor(self.TextActor)
        self.RenderWindow.Render()
        self.TextInputMode = 0
        if self.ExitTextInputCallback:
            self.ExitTextInputCallback(self.CurrentTextInput)
            self.ExitTextInputCallback = None
        if self.ExitAfterTextInputMode:
            self.RenderWindowInteractor.ExitCallback()

    def Render(self,interactive=1):
	
        if interactive:
            self.RenderWindowInteractor.Initialize()
        self.RenderWindow.SetWindowName("vmtk - the Vascular Modeling Toolkit")

        #sortedKeysStd = self.KeyBindingsStd.keys()
        #sortedKeysStd.sort()
        #textActorInputsStd = ['%s: %s' % (key, self.KeyBindingsStd[key]['text']) for key in sortedKeysStd]
        #self.TextActorStd.SetInput('\n'.join(textActorInputsStd))
        #self.Renderer.AddActor(self.TextActorStd)
    
        groups = list(set([self.KeyBindings[el]['group'] for el in self.KeyBindings]))
        groups.sort(reverse=True)

        textActorInputsList = []

        for group in groups:
            sortedKeys = [key for key in self.KeyBindings.keys() if self.KeyBindings[key]['group'] == group]
            sortedKeys.sort()
            textActorInputs = ['%s: %s' % (key, self.KeyBindings[key]['text']) for key in sortedKeys]
            textActorInputsList.append('\n'.join(textActorInputs))

        self.TextActor.SetInput('\n\n'.join(textActorInputsList))
        self.Renderer.AddActor(self.TextActor)

        #if len(self.KeyBindingsOpmode.keys()) != 0:
        #    sortedKeysOpmode = self.KeyBindingsOpmode.keys()
        #    sortedKeysOpmode.sort()
        #    textActorInputsOpmode = ['%s: %s' % (key, self.KeyBindingsOpmode[key]['text']) for key in sortedKeysOpmode]
        #    self.TextActorOpmode.SetInput('\n'.join(textActorInputsOpmode))
        #    self.TextActorOpmode.GetProperty().SetColor(1.0, 0.75, 0.32)
        #    self.Renderer.AddActor(self.TextActorOpmode)
        #else:
        #    self.TextActorOpmode.SetInput('.')
        #    self.Renderer.AddActor(self.TextActorOpmode)
 
        self.RenderWindow.Render()

        if interactive:
            self.RenderWindowInteractor.Start()

    def Initialize(self):

        if not self.Renderer:
            self.Renderer = vtk.vtkRenderer()
            self.Renderer.SetBackground(self.Background)
            self.RenderWindow = vtk.vtkRenderWindow()
            self.RenderWindow.AddRenderer(self.Renderer)
            self.RenderWindow.SetSize(self.WindowSize[0],self.WindowSize[1])
            self.RenderWindow.SetPointSmoothing(self.PointSmoothing)
            self.RenderWindow.SetLineSmoothing(self.LineSmoothing)
            self.RenderWindow.SetPolygonSmoothing(self.PolygonSmoothing)
            self.RenderWindowInteractor = vtk.vtkRenderWindowInteractor()
            if 'vtkCocoaRenderWindowInteractor' in dir(vtk) and vtk.vtkCocoaRenderWindowInteractor.SafeDownCast(self.RenderWindowInteractor):
                self.RenderWindowInteractor = vtkvmtk.vtkvmtkCocoaRenderWindowInteractor()
            self.RenderWindow.SetInteractor(self.RenderWindowInteractor)
            self.RenderWindowInteractor.SetInteractorStyle(vtkvmtk.vtkvmtkInteractorStyleTrackballCamera())
            self.RenderWindowInteractor.GetInteractorStyle().AddObserver("CharEvent",self.CharCallback)
            self.RenderWindowInteractor.GetInteractorStyle().AddObserver("KeyPressEvent",self.KeyPressCallback)

            self.AddKeyBinding('x','Take screenshot.',self.ScreenshotCallback,'0')
            self.AddKeyBinding('r','Reset camera.',self.ResetCameraCallback,'0')
            #self.AddKeyBinding('w','Show wireframe.',None,'0')
            #self.AddKeyBinding('r','Reset camera.',self.ResetCameraCallback, '0')
            #self.AddKeyBinding('s','Show surface.', None,'0')
            #self.AddKeyBinding('e','Quit renderer.',self.QuitRendererCallback,'0')
            self.AddKeyBinding('q','Quit renderer/proceed.',self.QuitRendererCallback,'0')
            #self.AddKeyBinding('3','3D.', None,'0')

            #self.TextActorStd = vtk.vtkTextActor()
            #self.TextActorStd.SetPosition(self.PositionStd)
            #self.Renderer.AddActor(self.TextActorStd)

            self.TextActor = vtk.vtkTextActor()
            self.TextActor.GetPositionCoordinate().SetCoordinateSystemToNormalizedViewport()
            self.TextActor.GetPosition2Coordinate().SetCoordinateSystemToNormalizedViewport()
            self.TextActor.SetPosition(self.Position)
            self.Renderer.AddActor(self.TextActor)

            #self.TextActorOpmode = vtk.vtkTextActor()
            #self.TextActorOpmode.SetPosition(self.PositionOpmode)
            #self.Renderer.AddActor(self.TextActorOpmode)

            self.TextInputActor = vtk.vtkTextActor()
            self.TextInputActor.GetPositionCoordinate().SetCoordinateSystemToNormalizedViewport()
            self.TextInputActor.GetPosition2Coordinate().SetCoordinateSystemToNormalizedViewport()
            self.TextInputActor.SetPosition(self.InputPosition)
 
        if self.UseRendererInputStream:
            self.InputStream = vmtkRendererInputStream(self)

    def RegisterScript(self, script):
        if self.UseRendererInputStream:
            script.InputStream = vmtkRendererInputStream(self)

    def Execute(self):
        self.Initialize()

    def Close(self,event,clientData):
        self.RenderWindowInteractor.Close()

    def Deallocate(self):
        if 'vtkCocoaRenderWindowInteractor' in dir(vtk) and vtkvmtk.vtkvmtkCocoaRenderWindowInteractor.SafeDownCast(self.RenderWindowInteractor):
            self.RenderWindowInteractor.AddObserver("TimerEvent", self.Close)
            self.RenderWindowInteractor.CreateOneShotTimer(1)
            self.RenderWindowInteractor.Start()
        self.RenderWindowInteractor = None
        self.RenderWindow = None
        self.Renderer = None
   
if __name__=='__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   PypeS
## Module:    $RCSfile: pype.py,v $
## Language:  Python
## Date:      $Date: 2006/07/07 10:45:42 $
## Version:   $Revision: 1.18 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENCE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even 
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
##      PURPOSE.  See the above copyright notices for more information.


import sys
from multiprocessing import Process, Manager
try:
    from vmtk import pypeserver
    from vmtk import pypes
except:

    import os

    if sys.platform == 'darwin':
        ldEnvironmentVariable = "DYLD_LIBRARY_PATH"

    elif sys.platform == 'win32':
        ldEnvironmentVariable = "PATH"

    else:
        ldEnvironmentVariable = "LD_LIBRARY_PATH"

    currentEnviron = dict()
    currentEnviron[ldEnvironmentVariable] = ""
    currentEnviron["PYTHONPATH"] = ""

    if os.environ.has_key(ldEnvironmentVariable):
        currentEnviron[ldEnvironmentVariable] = os.environ[ldEnvironmentVariable]

    if os.environ.has_key("PYTHONPATH"):
        currentEnviron["PYTHONPATH"] = os.environ["PYTHONPATH"]

    newEnviron = {}

    vmtkhome = os.path.dirname(os.path.abspath(__file__))

    if vmtkhome.endswith('bin'):
        vmtkhome = os.path.join(os.path.dirname(os.path.abspath(__file__)),"..")
    else:
        vmtkhome = os.path.join(os.path.dirname(os.path.abspath(__file__)),"..","..","..")
	
    vtkdir = [el for el in os.listdir(os.path.join(vmtkhome,"lib")) if el.startswith('vtk')][0]

    newEnviron[ldEnvironmentVariable] = os.path.join(vmtkhome,"bin") + os.path.pathsep + \
	                                os.path.join(vmtkhome,"lib",vtkdir) + os.path.pathsep + \
                                    os.path.join(vmtkhome,"lib","vmtk") + os.path.pathsep + \
                                    os.path.join(vmtkhome,"lib","InsightToolkit")

    os.environ[ldEnvironmentVariable] = newEnviron[ldEnvironmentVariable] + os.path.pathsep + currentEnviron[ldEnvironmentVariable]

    sys.path.append(os.path.join(vmtkhome,"bin","Python"))
    sys.path.append(os.path.join(vmtkhome,"lib",vtkdir))
    sys.path.append(os.path.join(vmtkhome,"lib","vmtk"))

    from vmtk import pypeserver
    from vmtk import pypes


if __name__=='__main__':
    manager = Manager()
    queue = manager.list()
    pypeProcess = Process(target=pypeserver.PypeServer, args=(queue,None), kwargs={"returnIfEmptyQueue":True})
    pypeProcess.start()

    args = sys.argv[:]
    if sys.argv[0].startswith('pyperun'):
        args = sys.argv[1:]

    queue.append(args)

    try:
        pypeProcess.join()
    except KeyboardInterrupt:
        pypeProcess.terminate()
    except BaseException, e:
        print e




""" This module loads all the classes from the vtkVmtk library into its
namespace.  This is a required module."""

import os
import vtk

if os.name == 'posix':
    from libvtkvmtkCommonPython import *
    from libvtkvmtkComputationalGeometryPython import *
    from libvtkvmtkDifferentialGeometryPython import *
    from libvtkvmtkIOPython import *
    from libvtkvmtkMiscPython import *
    from libvtkvmtkRenderingPython import *
    from libvtkvmtkSegmentationPython import *
    from libvtkvmtkITKPython import *
else:
    from vtkvmtkCommonPython import *
    from vtkvmtkComputationalGeometryPython import *
    from vtkvmtkDifferentialGeometryPython import *
    from vtkvmtkIOPython import *
    from vtkvmtkRenderingPython import *
    from vtkvmtkSegmentationPython import *
    from vtkvmtkITKPython import *
    



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtksurfaceclipper.py,v $
## Language:  Python
## Date:      $Date: 2006/05/26 12:35:13 $
## Version:   $Revision: 1.9 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENCE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even 
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
##      PURPOSE.  See the above copyright notices for more information.

import vtk
import sys

import vmtkrenderer
import pypes

vmtksurfaceclipper = 'vmtkSurfaceClipper'

class vmtkSurfaceClipper(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Surface = None
        self.vmtkRenderer = None
        self.OwnRenderer = 0

        self.WidgetType = 'box'

        self.Actor = None
        self.ClipWidget = None
        self.ClipFunction = None
        self.CleanOutput = 1
        self.Transform = None

        self.SetScriptName('vmtksurfaceclipper')
        self.SetScriptDoc('interactively clip a surface with a box')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','the input surface','vmtksurfacereader'],
            ['WidgetType','type','str',1,'["box","sphere"]','type of widget used for clipping'],
            ['Transform','transform','vtkTransform',1,'','the widget transform, useful in case of piping of multiple clipping scripts'],
            ['CleanOutput','cleanoutput','bool',1,'','toggle cleaning the unused points'],
            ['vmtkRenderer','renderer','vmtkRenderer',1,'','external renderer']
            ])
        self.SetOutputMembers([
            ['Surface','o','vtkPolyData',1,'','the output surface','vmtksurfacewriter'],
            ['Transform','otransform','vtkTransform',1,'','the output widget transform']
            ])

    def ClipCallback(self, obj):
        if self.ClipWidget.GetEnabled() != 1:
            return
        if self.WidgetType == "box":
            self.ClipWidget.GetPlanes(self.ClipFunction)
        elif self.WidgetType == "sphere":
            self.ClipWidget.GetSphere(self.ClipFunction)
        self.Clipper.Update()
        self.Surface.DeepCopy(self.Clipper.GetClippedOutput())
        self.Surface.Update()
        self.ClipWidget.Off()

    def InteractCallback(self):
        if self.BoxWidget.GetEnabled() == 1:
            self.BoxWidget.SetEnabled(0)
        else:
            self.BoxWidget.SetEnabled(1)

    def Display(self):

      	self.ClipWidget.SetInput(self.Surface)
      	self.ClipWidget.PlaceWidget()

        if self.Transform:
            self.ClipWidget.SetTransform(self.Transform)
            self.ClipWidget.On()
      	
        #self.vmtkRenderer.RenderWindowInteractor.Initialize()
        self.vmtkRenderer.Render()
        #self.vmtkRenderer.RenderWindowInteractor.Start()

    def Execute(self):

        if self.Surface == None:
            self.PrintError('Error: no Surface.')

        if self.WidgetType == "box":
            self.ClipFunction = vtk.vtkPlanes()
        elif self.WidgetType == "sphere":
            self.ClipFunction = vtk.vtkSphere()

        self.Clipper = vtk.vtkClipPolyData()
        self.Clipper.SetInput(self.Surface)
        self.Clipper.SetClipFunction(self.ClipFunction)
        self.Clipper.GenerateClippedOutputOn()
        self.Clipper.InsideOutOn()
        
        if not self.vmtkRenderer:
            self.vmtkRenderer = vmtkrenderer.vmtkRenderer()
            self.vmtkRenderer.Initialize()
            self.OwnRenderer = 1

        self.vmtkRenderer.RegisterScript(self) 

        mapper = vtk.vtkPolyDataMapper()
        mapper.SetInput(self.Surface)
        mapper.ScalarVisibilityOff()
        self.Actor = vtk.vtkActor()
        self.Actor.SetMapper(mapper)
        self.vmtkRenderer.Renderer.AddActor(self.Actor)

        if self.WidgetType == "box":
            self.ClipWidget = vtk.vtkBoxWidget()
            self.ClipWidget.GetFaceProperty().SetColor(0.6,0.6,0.2)
            self.ClipWidget.GetFaceProperty().SetOpacity(0.25)
        elif self.WidgetType == "sphere":
            self.ClipWidget = vtk.vtkSphereWidget()
            self.ClipWidget.GetSphereProperty().SetColor(0.6,0.6,0.2)
            self.ClipWidget.GetSphereProperty().SetOpacity(0.25)
            self.ClipWidget.GetSelectedSphereProperty().SetColor(0.6,0.0,0.0)
            self.ClipWidget.GetSelectedSphereProperty().SetOpacity(0.75)
            self.ClipWidget.SetRepresentationToSurface()
            self.ClipWidget.SetPhiResolution(20)
            self.ClipWidget.SetThetaResolution(20)

        self.ClipWidget.SetInteractor(self.vmtkRenderer.RenderWindowInteractor)
        
        self.vmtkRenderer.AddKeyBinding('space','Clip.',self.ClipCallback)
        self.vmtkRenderer.AddKeyBinding('i','Interact.',self.InteractCallback)
        self.Display()

        self.Transform = vtk.vtkTransform()
        self.ClipWidget.GetTransform(self.Transform)

        if self.OwnRenderer:
            self.vmtkRenderer.Deallocate()

        if self.CleanOutput == 1:
            cleaner = vtk.vtkCleanPolyData()
            cleaner.SetInput(self.Surface)
            cleaner.Update()
            self.Surface = cleaner.GetOutput()

        if self.Surface.GetSource():
            self.Surface.GetSource().UnRegisterAllOutputs()


if __name__=='__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkdijkstradistancetopoints.py,v $
## Language:  Python
## Date:      $$
## Version:   $$

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENCE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even 
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
##      PURPOSE.  See the above copyright notices for more information.

## Note: this class was contributed by 
##       Tangui Morvan
##       Kalkulo AS
##       Simula Research Laboratory

## This class computes the geodesic distance on the graph of a surface
## from a set of user-selected points on the surface using the Dijkstra algorithm.


import vtk
import sys

import vtkvmtk
import vtkvmtkcontrib
import vmtkrenderer
import pypes

vmtkdijkstradistancetopoints = 'vmtkDijkstraDistanceToPoints'


class vmtkDijkstraDistanceToPoints(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)
        
        self.Surface = None
        self.DijkstraDistanceToPointsArrayName = 'DijkstraDistanceToPoints'
        self.DistanceOffset = 0.
        self.DistanceScale = 1.
        self.MinDistance = 0.
        self.MaxDistance = -1.
        self.SeedPoints = vtk.vtkPolyData()
        self.SeedIds = vtk.vtkIdList()
        self.vmtkRenderer = None
        self.OwnRenderer = 0
        self.DisplayArray = False
        self.SurfaceMapper = None
        self.Opacity = 1.
        self.PointActor = None
        self.ScalarBarActor = None
        self.InteractionMode = 0
        self.ExamineSurface = None
        self.ExamineSpheres = vtk.vtkPolyData()
        self.ExamineSpheresActor = None
        self.ExamineText = None

        self.SetScriptName('vmtkdijkstradistancetopoints')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','the input surface','vmtksurfacereader'],
            ['DijkstraDistanceToPointsArrayName','distancetopointsarray','str',1,'','array storing the distances'],
            ['DistanceOffset','offset','float',1,'','offset added to the distances'],
            ['DistanceScale','scale','float',1,'','scale applied to the distances'],
            ['MinDistance','mindistance','float',1,'','minimum value for the distances'],
            ['MaxDistance','maxdistance','float',1,'','maximum value for the distances'],
            ['Opacity','opacity','float',1,'(0.0,1.0)','object opacities in the scene'],
            ['vmtkRenderer','renderer','vmtkRenderer',1,'','external renderer']
            ])
        self.SetOutputMembers([
            ['Surface','o','vtkPolyData',1,'','','vmtksurfacewriter']
            ])
    
    def DistanceParametersValidator(self,text):
        if not text:
            return 1
        splitText = text.strip().split(' ')
        if len(splitText) not in range(1,5):
            return 0
        try:
            for i in range(1,len(splitText)+1):
                float(splitText[i-1])
        except ValueError:
            return 0
        return 1
        
        
    def ComputeDistances(self):
        dijkstraFilter = vtkvmtkcontrib.vtkvmtkPolyDataDijkstraDistanceToPoints()
        dijkstraFilter.SetInput(self.Surface)
        dijkstraFilter.SetSeedIds(self.SeedIds)
        dijkstraFilter.SetDistanceOffset(self.DistanceOffset)
        dijkstraFilter.SetDistanceScale(self.DistanceScale)
        dijkstraFilter.SetMinDistance(self.MinDistance)
        dijkstraFilter.SetMaxDistance(self.MaxDistance)
        dijkstraFilter.SetDijkstraDistanceToPointsArrayName(self.DijkstraDistanceToPointsArrayName)
        dijkstraFilter.Update()
        return dijkstraFilter.GetOutput()
    
    def InitializeSeeds(self):
        if (self.InteractionMode==0):
	    self.SeedIds.Initialize()
	    self.SeedPoints.Initialize()
	    seedPoints = vtk.vtkPoints()
	    self.SeedPoints.SetPoints(seedPoints)
	else:
	    self.ExamineSpheres.Initialize()
	    spherePoints = vtk.vtkPoints()
	    self.ExamineSpheres.SetPoints(spherePoints)
	    self.ExamineSpheres.GetPointData().Initialize()
	    sphereRadii = vtk.vtkDoubleArray()
	    self.ExamineSpheres.GetPointData().SetScalars(sphereRadii)
        
    def UndoCallback(self, obj):
        self.InitializeSeeds()
        self.SeedPoints.Modified()
        self.vmtkRenderer.RenderWindow.Render()

    def SpaceCallback(self,obj):
        picker = vtk.vtkCellPicker()
        picker.SetTolerance(1E-4 * self.Surface.GetLength())
        eventPosition = self.vmtkRenderer.RenderWindowInteractor.GetEventPosition()
        #eventPosition = obj.GetEventPosition()
        result = picker.Pick(float(eventPosition[0]),float(eventPosition[1]),0.0,self.vmtkRenderer.Renderer)
        if result == 0:
            return
        pickPosition = picker.GetPickPosition()
        pickedCellPointIds = self.Surface.GetCell(picker.GetCellId()).GetPointIds()
        minDistance = 1E10
        pickedSeedId = -1
        for i in range(pickedCellPointIds.GetNumberOfIds()):
            distance = vtk.vtkMath.Distance2BetweenPoints(pickPosition,self.Surface.GetPoint(pickedCellPointIds.GetId(i)))
            if distance < minDistance:
                minDistance = distance
                pickedSeedId = pickedCellPointIds.GetId(i)
        if pickedSeedId == -1:
            pickedSeedId = pickedCellPointIds.GetId(0)
        if (self.InteractionMode==0):
            self.SeedIds.InsertNextId(pickedSeedId)
            point = self.Surface.GetPoint(pickedSeedId)
            self.SeedPoints.GetPoints().InsertNextPoint(point)
            self.SeedPoints.Modified()
        else:
            point = self.Surface.GetPoint(pickedSeedId)
            self.ExamineSpheres.GetPoints().InsertNextPoint(point)
            length = 0.
            array = self.ExamineSurface.GetPointData().GetArray(self.DijkstraDistanceToPointsArrayName)
            if array:
                length = array.GetComponent(pickedSeedId,0)
            self.ExamineSpheres.GetPointData().GetScalars().InsertNextValue(length)
            self.ExamineSpheres.Modified()
        self.vmtkRenderer.RenderWindow.Render()

    def DisplayDistanceCallback(self,obj):
        self.DisplayArray = not self.DisplayArray
        if self.DisplayArray:
            newSurface = self.ComputeDistances()
            self.SurfaceMapper.SetInput(newSurface)
            newSurface.GetPointData().SetActiveScalars(self.DijkstraDistanceToPointsArrayName)
            array = newSurface.GetPointData().GetScalars()
            if array:
                self.SurfaceMapper.SetScalarRange(array.GetRange(0))
                self.ScalarBarActor.VisibilityOn()
        else:
            self.SurfaceMapper.SetInput(self.Surface)
            self.ScalarBarActor.VisibilityOff()
        self.SurfaceMapper.SetScalarVisibility(self.DisplayArray)
        self.vmtkRenderer.RenderWindow.Render()

    def AddCallback(self, obj):
	    queryString = 'Please input new parameters :\nDistanceOffset('+str(self.DistanceOffset)+') [DistanceScale('+str(self.DistanceScale)+') MinDistance('+str(self.MinDistance)+') MaxDistance('+str(self.MaxDistance)+'): '
            inputString = self.InputText(queryString,self.DistanceParametersValidator)
            splitInputString = inputString.strip().split(' ')
            if len(splitInputString) >= 1 and splitInputString[0] != '':
                self.DistanceOffset = float(splitInputString[0])
            if len(splitInputString) >= 2:
                self.DistanceScale = float(splitInputString[1])
            if len(splitInputString) >= 3:
                self.MinDistance = float(splitInputString[2])
            if len(splitInputString) >= 4:
                self.MaxDistance = float(splitInputString[3])

    def ExamineCallback(self, obj):
	    #Switch beetween examien and interact mode
      if self.InteractionMode == 0:
          self.InteractionMode = 1
          self.ExamineSurface = self.ComputeDistances()
          self.PointActor.VisibilityOff()
          self.ExamineSpheresActor.VisibilityOn()
          self.ExamineText.VisibilityOn()
          self.InitializeSeeds()
      else:
          self.InteractionMode = 0
          #Compute the distances
          self.PointActor.VisibilityOn()
          self.ExamineSpheresActor.VisibilityOff()
          self.ExamineText.VisibilityOff()
      self.vmtkRenderer.RenderWindow.Render()

    def Execute(self):

        if self.Surface == None:
            self.PrintError('Error: No input surface.')

        if not self.vmtkRenderer:
          self.vmtkRenderer = vmtkrenderer.vmtkRenderer()
          self.vmtkRenderer.Initialize()
          self.OwnRenderer = 1
          

        self.vmtkRenderer.RegisterScript(self) 
	
        glyphs = vtk.vtkGlyph3D()
        glyphSource = vtk.vtkSphereSource()
        glyphSource.SetRadius(1)
        glyphs.SetInput(self.SeedPoints)
        glyphs.SetSource(glyphSource.GetOutput())
        glyphs.SetScaleModeToDataScalingOff()
        glyphs.SetScaleFactor(self.Surface.GetLength()*0.01)
        glyphMapper = vtk.vtkPolyDataMapper()
        glyphMapper.SetInput(glyphs.GetOutput())
        glyphMapper.ScalarVisibilityOff()
        self.PointActor = vtk.vtkActor()
        self.PointActor.SetMapper(glyphMapper)
        self.PointActor.GetProperty().SetColor(1.0,0.0,0.0)
        self.PointActor.GetProperty().SetOpacity(self.Opacity)
        self.PointActor.PickableOff()
        self.vmtkRenderer.Renderer.AddActor(self.PointActor)
        
        examineGlyphs = vtk.vtkGlyph3D()
        examineGlyphSource = vtk.vtkSphereSource()
        examineGlyphSource.SetRadius(1)
        examineGlyphs.SetInput(self.ExamineSpheres)
        examineGlyphs.SetSource(examineGlyphSource.GetOutput())
        examineGlyphs.SetScaleModeToScaleByScalar()
        examineGlyphs.SetScaleFactor(1.)
        examineGlyphMapper = vtk.vtkPolyDataMapper()
        examineGlyphMapper.SetInput(examineGlyphs.GetOutput())
        examineGlyphMapper.ScalarVisibilityOff()
        self.ExamineSpheresActor = vtk.vtkActor()
        self.ExamineSpheresActor.SetMapper(examineGlyphMapper)
        self.ExamineSpheresActor.GetProperty().SetColor(0.0,1.0,0.0)
        self.ExamineSpheresActor.GetProperty().SetOpacity(self.Opacity)
        self.ExamineSpheresActor.PickableOff()
        self.ExamineSpheresActor.VisibilityOff()
        self.vmtkRenderer.Renderer.AddActor(self.ExamineSpheresActor)
	
        #self.vmtkRenderer.RenderWindowInteractor.AddObserver("KeyPressEvent", self.KeyPressed)
        self.vmtkRenderer.AddKeyBinding('u','Undo.',self.UndoCallback)
        self.vmtkRenderer.AddKeyBinding('space','Pick points.',self.SpaceCallback)
        self.vmtkRenderer.AddKeyBinding('w','Examine mode.',self.ExamineCallback)
        self.vmtkRenderer.AddKeyBinding('d','Display Distance.',self.DisplayDistanceCallback)
        self.vmtkRenderer.AddKeyBinding('a','Add.',self.AddCallback)
        
        self.SurfaceMapper = vtk.vtkPolyDataMapper()
        self.SurfaceMapper.SetInput(self.Surface)
        self.SurfaceMapper.SetScalarVisibility(self.DisplayArray)
        surfaceActor = vtk.vtkActor()
        surfaceActor.SetMapper(self.SurfaceMapper)
        surfaceActor.GetProperty().SetOpacity(self.Opacity)
        self.vmtkRenderer.Renderer.AddActor(surfaceActor)
        
        self.ScalarBarActor = vtk.vtkScalarBarActor()
        self.ScalarBarActor.SetLookupTable(self.SurfaceMapper.GetLookupTable())
        self.ScalarBarActor.GetLabelTextProperty().ItalicOff()
        self.ScalarBarActor.GetLabelTextProperty().BoldOff()
        self.ScalarBarActor.GetLabelTextProperty().ShadowOff()
        self.ScalarBarActor.SetLabelFormat('%.2f')
        self.ScalarBarActor.SetTitle('distances')
        self.ScalarBarActor.VisibilityOff()
        self.vmtkRenderer.Renderer.AddActor(self.ScalarBarActor)
        
        self.ExamineText = vtk.vtkTextActor()
        self.ExamineText.SetInput("Examine Mode")
        self.ExamineText.GetPositionCoordinate().SetCoordinateSystemToNormalizedViewport()
        self.ExamineText.SetPosition(0.05,0.95)
        self.ExamineText.VisibilityOff()
        self.vmtkRenderer.Renderer.AddActor2D(self.ExamineText)
        
        self.InputInfo('Please position the mouse and press space to add points, \'u\' to undo\n')

        any = 0
        while any == 0:
            self.InitializeSeeds()
            self.vmtkRenderer.Render()
            any = self.SeedIds.GetNumberOfIds()
 
        self.Surface = self.ComputeDistances()

        if self.Surface.GetSource():
            self.Surface.GetSource().UnRegisterAllOutputs()

        if self.OwnRenderer:
            self.vmtkRenderer.Deallocate()

if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



__all__ = [
  'vmtkactivetubes',
  'vmtkbifurcationprofiles',
  'vmtkbifurcationreferencesystems',
  'vmtkbifurcationsections',
  'vmtkbifurcationvectors',
  'vmtkboundarylayer',
  'vmtkboundaryreferencesystems',
  'vmtkbranchclipper',
  'vmtkbranchextractor',
  'vmtkbranchgeometry',
  'vmtkbranchmapping',
  'vmtkbranchmetrics',
  'vmtkbranchpatching',
  'vmtkbranchsections',
  'vmtkcenterlineattributes',
  'vmtkcenterlinegeometry',
  'vmtkcenterlinelabeler',
  'vmtkcenterlinemerge',
  'vmtkcenterlinemodeller',
  'vmtkcenterlineoffsetattributes',
  'vmtkcenterlineresampling',
  'vmtkcenterlines',
  'vmtkcenterlinesections',
  'vmtkcenterlinesmoothing',
  'vmtkcenterlineviewer',
  'vmtkdelaunayvoronoi',
  'vmtkdistancetocenterlines',
  'vmtkendpointextractor',
  'vmtkflowextensions',
  'vmtkicpregistration',
  'vmtkimagecast',
  'vmtkimagecompose',
  'vmtkimagecurvedmpr',
  'vmtkimagefeaturecorrection',
  'vmtkimagefeatures',
  'vmtkimageinitialization',
  'vmtkimagelinetracer',
  'vmtkimagemipviewer',
  'vmtkimageobjectenhancement',
  'vmtkimagereader',
  'vmtkimagereslice',
  'vmtkimageseeder',
  'vmtkimageshiftscale',
  'vmtkimagesmoothing',
  'vmtkimageviewer',
  'vmtkimagevesselenhancement',
  'vmtkimagevoipainter',
  'vmtkimagevoiselector',
  'vmtkimagewriter',
  'vmtklevelsetsegmentation',
  'vmtklineartoquadratic',
  'vmtklineresampling',
  'vmtklocalgeometry',
  'vmtkmarchingcubes',
  'vmtkmeshboundaryinspector',
  'vmtkmeshbranchclipper',
  'vmtkmeshclipper',
  'vmtkmeshdatareader',
  'vmtkmeshlambda2',
  'vmtkmeshlinearize',
  'vmtkmeshgenerator',
  'vmtkmeshprojection',
  'vmtkmeshreader',
  'vmtkmeshscaling',
  'vmtkmeshtetrahedralize',
  'vmtkmeshtosurface',
  'vmtkmeshtransform',
  'vmtkmeshtransformtoras',
  'vmtkmeshvectorfromcomponents',
  'vmtkmeshviewer',
  'vmtkmeshvorticityhelicity',
  'vmtkmeshwallshearrate',
  'vmtkmeshwriter',
  'vmtknetworkeditor',
  'vmtknetworkextraction',
  'vmtknetworkwriter',
  'vmtkpointsplitextractor',
  'vmtkpointtransform',
  'vmtkpolyballmodeller',
  'vmtkpotentialfit',
  'vmtkpythonscript',
  'vmtkrenderer',
  'vmtkrendertoimage',
  'vmtkrbfinterpolation',
  'vmtksurfaceappend',
  'vmtksurfacecapper',
  'vmtksurfacecelldatatopointdata',
  'vmtksurfacecenterlineprojection',
  'vmtksurfaceclipper',
  'vmtksurfaceconnectivity',
  'vmtksurfacedecimation',
  'vmtksurfacedistance',
  'vmtksurfacekiteremoval',
  'vmtksurfacemodeller',
  'vmtksurfacenormals',
  'vmtksurfacepointdatatocelldata',
  'vmtksurfaceprojection',
  'vmtksurfacereader',
  'vmtksurfacereferencesystemtransform',
  'vmtksurfaceremeshing',
  'vmtksurfacescaling',
  'vmtksurfacesmoothing',
  'vmtksurfacesubdivision',
  'vmtksurfacetransform',
  'vmtksurfacetransforminteractive',
  'vmtksurfacetransformtoras',
  'vmtksurfacetriangle',
  'vmtksurfacetomesh',
  'vmtksurfaceviewer',
  'vmtksurfacewriter',
  'vmtksurfmesh',
  'vmtktetgen',
  'vmtktetringenerator'
  ]

for item in __all__:
        exec('from '+item+' import *')




#!/usr/bin/env python
                                                                                                                                                        
__all__ = [
    'pypescript',
    'pype',
    'pypebatch'
    ]

for item in __all__:
    exec('from '+item+' import *')




#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtknetworkeditor.py,v $
## Language:  Python
## Date:      $Date: 2006/05/26 12:35:13 $
## Version:   $Revision: 1.3 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENCE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even 
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
##      PURPOSE.  See the above copyright notices for more information.

##   Developed with support from the EC FP7/2007-2013: ARCH, Project n. 224390

import vtk
import sys

import vtkvmtk
import vmtkrenderer
import vmtkimageviewer
import vmtkimagefeatures
import pypes

import vmtkactivetubes

vmtknetworkeditor = 'vmtkNetworkEditor'

class vmtkNetworkEditor(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)
        
        self.Network = None
        self.RadiusArrayName = 'Radius'
        self.LabelsArrayName = 'Labels'

        self.SplineInterpolation = 1

        self.UseActiveTubes = 0
        self.NumberOfIterations = 100
        self.PotentialWeight = 1.0
        self.StiffnessWeight = 0.0

        self.NetworkTube = None

        self.Selection = None
        self.SelectionPoints = None
        self.SelectionRadiusArray = None
        self.SelectionActor = None
        self.SelectedCellId = -1
        self.SelectedSubId = -1
        self.SelectedPCoords = 0.0

        self.ActiveSegment = None
        self.ActiveSegmentPoints = None
        self.ActiveSegmentCellArray = None
        self.ActiveSegmentRadiusArray = None
        self.ActiveSegmentActor = None

        self.ActiveSegmentSeeds = None
        self.ActiveSegmentSeedsPoints = None
        self.ActiveSegmentSeedsRadiusArray = None
        self.ActiveSegmentSeedsActor = None

        self.AttachedCellIds = [-1,-1]
        self.AttachedSubIds = [-1,-1]
        self.AttachedPCoords = [0.0,0.0]

        self.PlaneWidgetX = None
        self.PlaneWidgetY = None
        self.PlaneWidgetZ = None

        self.Image = None
        self.FeatureImage = None

        self.CurrentRadius = 0.0

        self.NetworkRadiusArray = None
        self.NetworkLabelsArray = None

        self.CellPicker = None

        self.CellIdsToMerge = []

        self.OperationMode = None
        self.PickMode = 'image'

        self.vmtkRenderer = None
        self.OwnRenderer = 0

        self.SetScriptName('vmtknetworkeditor')
        self.SetScriptDoc('')
        self.SetInputMembers([
            ['Network','i','vtkPolyData',1,'','the input network','vmtksurfacereader'],
            ['RadiusArrayName','radiusarray','str',1,''],
            ['LabelsArrayName','labelsarray','str',1,''],
            ['SplineInterpolation','spline','bool',1,''],
            ['UseActiveTubes','activetubes','bool',1,''],
            ['NumberOfIterations','iterations','int',1,'(0,)'],
            ['PotentialWeight','potentialweight','float',1,'(0.0,)'],
            ['StiffnessWeight','stiffnessweight','float',1,'(0.0,)'],
            ['PlaneWidgetX','xplane','vtkImagePlaneWidget',1,'','the X image plane widget'],
            ['PlaneWidgetY','yplane','vtkImagePlaneWidget',1,'','the Y image plane widget'],
            ['PlaneWidgetZ','zplane','vtkImagePlaneWidget',1,'','the Z image plane widget'],
            ['Image','image','vtkImageData',1,'','','vmtkimagereader'],
            ['FeatureImage','featureimage','vtkImageData',1,'','','vmtkimagereader'],
            ['vmtkRenderer','renderer','vmtkRenderer',1,'','external renderer']])
        self.SetOutputMembers([
            ['Network','o','vtkPolyData',1,'','the output network','vmtknetworkwriter'],
            ['RadiusArrayName','oradiusarray','str',1,''],
            ['Surface','osurface','vtkPolyData',1,'','the output surface','vmtksurfacewriter']])

    def TogglePickMode(self):
        if self.PickMode == 'image':
            self.PickMode = 'network'
        else:
            self.PickMode = 'image'

    def TogglePlaneWidget(self,planeWidget):
        if not planeWidget:
            return
        interaction = planeWidget.GetInteraction()
        if interaction:
            planeWidget.InteractionOff()
        else:
            planeWidget.InteractionOn()

    def UpdateLabels(self):
        self.CellCenters.Modified()
        self.CellCenters.Update()
        self.Render()

    def ToggleLabels(self):
        if self.LabelsActor.GetVisibility() == 1:
            self.LabelsActor.VisibilityOff()
        else:
            self.LabelsActor.VisibilityOn()
        self.Render()

    def SetPickMode(self,pickMode):
        if pickMode == 'image':
            self.PickMode = 'image'
            if self.PlaneWidgetX:
                self.PlaneWidgetX.InteractionOn()
            if self.PlaneWidgetY:
                self.PlaneWidgetY.InteractionOn()
            if self.PlaneWidgetZ:
                self.PlaneWidgetZ.InteractionOn()
        elif pickMode == 'network':
            self.PickMode = 'network'
            if self.PlaneWidgetX:
                self.PlaneWidgetX.InteractionOff()
            if self.PlaneWidgetY:
                self.PlaneWidgetY.InteractionOff()
            if self.PlaneWidgetZ:
                self.PlaneWidgetZ.InteractionOff()

    def KeyReleaseCallback(self,obj,event):
        pass
        #key = object.GetKeySym()
        #if self.OperationMode == 'add':
        #    if key == 'Shift_L' or key == 'Shift_R':
        #        if self.PickMode == 'network':
        #            self.SetPickMode('image')
    
    def CheckMenu(self):
        self.vmtkRenderer.RemoveKeyBinding('space')
        self.vmtkRenderer.RemoveKeyBinding('c')
        self.vmtkRenderer.RemoveKeyBinding('u')
        self.vmtkRenderer.RemoveKeyBinding('+')
        self.vmtkRenderer.RemoveKeyBinding('-')
        self.vmtkRenderer.RemoveKeyBinding('=')
        self.vmtkRenderer.RemoveKeyBinding('Return')

    def ShowLabelCallback(self, obj):
        self.ToggleLabels()

    def AddCallback(self, obj):
        self.InputInfo('Switched to add mode.\nCtrl + left click to add tubes.')
        #self.PrintLog('Add mode')
        self.OperationMode = 'add'
        self.InitializeActiveSegment()
        self.vmtkRenderer.AddKeyBinding('space','Toggle image/tube interaction',self.SpaceCallback, '2')
        self.vmtkRenderer.AddKeyBinding('c','Cancel',self.CancelCallback,'2')
        self.vmtkRenderer.AddKeyBinding('u','Undo',self.UndoCallback,'2')
        self.vmtkRenderer.AddKeyBinding('+','Increase radius',self.PlusCallback,'2')
        self.vmtkRenderer.AddKeyBinding('=','Increase radius',self.PlusCallback,'2')
        self.vmtkRenderer.AddKeyBinding('-','Decrease radius',self.MinusCallback,'2')
        self.vmtkRenderer.AddKeyBinding('Return','Accept tube',self.ReturnAddCallback,'2')
        self.UpdateAndRender()

    def DeleteCallback(self, obj):
        self.InputInfo('Switched to delete mode.\nCtrl + left click to delete tubes.')
        #self.PrintLog('Delete mode')
        self.OperationMode = 'delete'
        self.InitializeActiveSegment()
        self.CheckMenu()
        if self.PickMode == 'image':
            self.SetPickMode('network')
            self.InitializeActiveSegment()
        self.UpdateAndRender()

    def SplitCallback(self, obj):
        self.InputInfo('Switched to split mode.\nCtrl + left click to split tubes.')
        #self.PrintLog('Split mode')
        self.OperationMode = 'split'
        self.InitializeActiveSegment()
        self.CheckMenu()
        if self.PickMode == 'image':
            self.SetPickMode('network')
            self.InitializeActiveSegment()
        self.UpdateAndRender()

    def LabelCallback(self, obj):
        #self.PrintLog('Label mode')
        self.InputInfo('Switched to label mode.\nCtrl + left click to add label.')
        self.OperationMode = 'label'
        self.InitializeActiveSegment()
        self.CheckMenu()
        if self.PickMode == 'image':
            self.SetPickMode('network')
            self.InitializeActiveSegment()
        self.UpdateAndRender()

    def MergeCallback(self, obj):
        self.InputInfo('Switched to merge mode.\nCtrl + left click to select two tubes to merge.')
        #self.PrintLog('Merge mode')
        self.OperationMode = 'merge'
        self.InitializeActiveSegment()
        self.CheckMenu()
        if self.PickMode == 'image':
            self.SetPickMode('network')
            self.InitializeActiveSegment()
            self.CellIdsToMerge = []
        self.vmtkRenderer.AddKeyBinding('Return','Accept merge',self.ReturnCallback,'2')
        self.UpdateAndRender()

    def SpaceCallback(self, obj):
        self.TogglePickMode()
        self.TogglePlaneWidget(self.PlaneWidgetX)
        self.TogglePlaneWidget(self.PlaneWidgetY)
        self.TogglePlaneWidget(self.PlaneWidgetZ)
        self.InputInfo('Ctrl + left click to add seeds.')	

    def CancelCallback(self, obj):
        self.InitializeActiveSegment()

    def UndoCallback(self, obj):
        numberOfSeeds = self.ActiveSegmentSeedsPoints.GetNumberOfPoints()
        if numberOfSeeds > 0:
            self.ActiveSegmentSeedsPoints.SetNumberOfPoints(numberOfSeeds-1)
            self.ActiveSegmentSeedsRadiusArray.SetNumberOfValues(numberOfSeeds-1)
            self.ActiveSegmentSeeds.Modified()
            self.AttachedCellIds[1] = -1
            self.AttachedSubIds[1] = -1
            self.AttachedPCoords[1] = 0.0
        else:
            self.AttachedCellIds[0] = -1
            self.AttachedSubIds[0] = -1
            self.AttachedPCoords[0] = 0.0
        numberOfPoints = self.ActiveSegmentPoints.GetNumberOfPoints()
        if numberOfPoints > 2:
            self.ActiveSegmentPoints.SetNumberOfPoints(numberOfPoints-1)
            self.ActiveSegmentRadiusArray.SetNumberOfValues(numberOfPoints-1)
            self.ActiveSegmentCellArray.Initialize()
            self.ActiveSegmentCellArray.InsertNextCell(numberOfPoints-1)
            for i in range(numberOfPoints-1):
                self.ActiveSegmentCellArray.InsertCellPoint(i)
            self.ActiveSegment.Modified()
        else:
            self.ActiveSegmentPoints.SetNumberOfPoints(0)
            self.ActiveSegmentCellArray.Initialize()
            self.ActiveSegmentRadiusArray.SetNumberOfValues(0)
            self.ActiveSegment.Modified()
        self.UpdateAndRender()

    def PlusCallback(self, obj):
        numberOfSeeds = self.ActiveSegmentSeedsPoints.GetNumberOfPoints()
        if numberOfSeeds > 0:
            radius = self.ActiveSegmentSeedsRadiusArray.GetValue(numberOfSeeds-1)
            radius += 0.1
            self.ActiveSegmentSeedsRadiusArray.SetValue(numberOfSeeds-1,radius)
            self.CurrentRadius = radius
            self.ActiveSegmentSeeds.Modified()
        numberOfPoints = self.ActiveSegmentPoints.GetNumberOfPoints()
        if numberOfPoints > 0:
            radius = self.ActiveSegmentRadiusArray.GetValue(numberOfPoints-1)
            radius += 0.1
            self.ActiveSegmentRadiusArray.SetValue(numberOfPoints-1,radius)
            self.ActiveSegment.Modified()
        self.UpdateAndRender()

    def MinusCallback(self, obj):
        numberOfSeeds = self.ActiveSegmentSeedsPoints.GetNumberOfPoints()
        if numberOfSeeds > 0:
            radius = self.ActiveSegmentSeedsRadiusArray.GetValue(numberOfSeeds-1)
            radius -= 0.1
            if radius > 0.0:
                self.ActiveSegmentSeedsRadiusArray.SetValue(numberOfSeeds-1,radius)
                self.CurrentRadius = radius
                self.ActiveSegmentSeeds.Modified()
        numberOfPoints = self.ActiveSegmentPoints.GetNumberOfPoints()
        if numberOfPoints > 0:
            radius = self.ActiveSegmentRadiusArray.GetValue(numberOfPoints-1)
            radius -= 0.1
            if radius > 0.0:
                self.ActiveSegmentRadiusArray.SetValue(numberOfPoints-1,radius)
                self.ActiveSegment.Modified()
        self.UpdateAndRender()

    def ReturnAddCallback(self, obj):
        attachedPointId0 = -1
        attachedPointId1 = -1
        cellIdsToRemove = []
        if self.AttachedCellIds[0] != -1:
            attachedPointId0, hasSplit = self.SplitCellNoRemove(self.AttachedCellIds[0],self.AttachedSubIds[0],self.AttachedPCoords[0])
            if hasSplit:
                cellIdsToRemove.append(self.AttachedCellIds[0])
        if self.AttachedCellIds[1] != -1:
            attachedPointId1, hasSplit = self.SplitCellNoRemove(self.AttachedCellIds[1],self.AttachedSubIds[1],self.AttachedPCoords[1])
            if hasSplit:
                cellIdsToRemove.append(self.AttachedCellIds[1])
        if cellIdsToRemove:
            self.RemoveCells(cellIdsToRemove)
        segment = self.ActiveSegmentActor.GetMapper().GetInput()
        if self.UseActiveTubes:
            self.RunActiveTube(segment)
        segmentRadiusArray = segment.GetPointData().GetArray(self.RadiusArrayName)
        numberOfSegmentPoints = segment.GetNumberOfPoints()
        networkPoints = self.Network.GetPoints()
        networkCellArray = self.Network.GetLines()
        cellId = networkCellArray.InsertNextCell(numberOfSegmentPoints)
        for i in range(numberOfSegmentPoints):
            id = -1
            if i == 0 and attachedPointId0 != -1:
                id = attachedPointId0
            elif i == numberOfSegmentPoints-1 and attachedPointId1 != -1:
                id = attachedPointId1
            else:
                id = networkPoints.InsertNextPoint(segment.GetPoint(i))
                self.NetworkRadiusArray.InsertTuple1(id,segmentRadiusArray.GetTuple1(i))
            networkCellArray.InsertCellPoint(id)
        self.NetworkLabelsArray.InsertValue(cellId,'')
        self.Network.BuildCells()
        self.Network.Modified()
        self.NetworkTube.Modified()
        self.UpdateLabels()
        #if self.Image and self.PickMode == 'network':
        #    self.SetPickMode('image')
        self.InitializeSelection()
        self.InitializeActiveSegment()

    def ReturnCallback(self, obj):
        numberOfActiveCells = self.ActiveSegment.GetNumberOfCells()
        if numberOfActiveCells != 2 or len(self.CellIdsToMerge) != 2:
            return
        cell0ToMerge = self.Network.GetCell(self.CellIdsToMerge[0])
        cell0PointIds = vtk.vtkIdList()
        cell0PointIds.DeepCopy(cell0ToMerge.GetPointIds())
        numberOfCell0Points = cell0PointIds.GetNumberOfIds()
        cell1ToMerge = self.Network.GetCell(self.CellIdsToMerge[1])
        cell1PointIds = vtk.vtkIdList()
        cell1PointIds.DeepCopy(cell1ToMerge.GetPointIds())
        numberOfCell1Points = cell1PointIds.GetNumberOfIds()
        reverse = [False,False] 
        if cell0PointIds.GetId(numberOfCell0Points-1) == cell1PointIds.GetId(0):
            reverse = [False,False]
        elif cell0PointIds.GetId(numberOfCell0Points-1) == cell1PointIds.GetId(numberOfCell1Points-1):
            reverse = [False,True]
        elif cell0PointIds.GetId(0) == cell1PointIds.GetId(numberOfCell1Points-1):
            reverse = [True,True]
        elif cell0PointIds.GetId(0) == cell1PointIds.GetId(0):
            reverse = [True,False]
        else:
            self.PrintLog('Error: trying to merge non-adjacent segments.')
            return
        mergedLabel = self.NetworkLabelsArray.GetValue(self.CellIdsToMerge[0])
        self.RemoveCells(self.CellIdsToMerge)
        networkCellArray = self.Network.GetLines()
        cellId = networkCellArray.InsertNextCell(numberOfCell0Points+numberOfCell1Points-1)
        for i in range(numberOfCell0Points):
            loc = i
            if reverse[0]:
                loc = numberOfCell0Points - 1 - i
            networkCellArray.InsertCellPoint(cell0PointIds.GetId(loc))
        for i in range(1,numberOfCell1Points):
            loc = i
            if reverse[1]:
                loc = numberOfCell1Points - 1 - i
            networkCellArray.InsertCellPoint(cell1PointIds.GetId(loc))
        self.NetworkLabelsArray.InsertValue(cellId,mergedLabel)
        self.Network.SetLines(networkCellArray)
        self.Network.BuildCells()
        self.Network.Modified()
        self.NetworkTube.Modified()
        self.UpdateLabels()
        self.InitializeActiveSegment()
        self.CellIdsToMerge = []

    def RemoveCell(self,cellId):
        self.RemoveCells([cellId])

    def RemoveCells(self,cellIds):
        networkCellArray = vtk.vtkCellArray()
        networkLabelsArray = vtk.vtkStringArray()
        numberOfCells = self.Network.GetNumberOfCells()
        for i in range(numberOfCells):
            if i in cellIds:
                continue
            networkCellArray.InsertNextCell(self.Network.GetCell(i))
            networkLabelsArray.InsertNextValue(self.NetworkLabelsArray.GetValue(i))
        self.Network.SetLines(networkCellArray)
        self.NetworkLabelsArray.DeepCopy(networkLabelsArray)
        self.Network.BuildCells()
        self.Network.Modified()

    def SplitCell(self,cellId,subId,pcoord):
        splitPointId, hasSplit = self.SplitCellNoRemove(cellId,subId,pcoord)
        if hasSplit:
            self.RemoveCell(cellId)
        return splitPointId, hasSplit

    def SplitCellNoRemove(self,cellId,subId,pcoord):
        splitId = subId
        if pcoord > 0.5:
            splitId = subId+1
        cell = self.Network.GetCell(cellId)
        label = self.NetworkLabelsArray.GetValue(cellId)
        numberOfCellPoints = cell.GetNumberOfPoints()
        cellPointIds = cell.GetPointIds()
        splitPointId = cellPointIds.GetId(splitId)
        if splitId == 0 or splitId == numberOfCellPoints-1:
            return splitPointId, False
        lines = self.Network.GetLines()
        newCellId1 = lines.InsertNextCell(splitId+1)
        for i in range(splitId+1):
            lines.InsertCellPoint(cellPointIds.GetId(i))
        newCellId2 = lines.InsertNextCell(numberOfCellPoints-splitId)
        for i in range(splitId,numberOfCellPoints):
            lines.InsertCellPoint(cellPointIds.GetId(i))
        newLabel1 = ''
        newLabel2 = ''
        if label:
            newLabel1 = label + "1"
            newLabel2 = label + "2"
        self.NetworkLabelsArray.InsertValue(newCellId1,newLabel1)
        self.NetworkLabelsArray.InsertValue(newCellId2,newLabel2)
        self.Network.BuildCells()
        return splitPointId, True

    def LeftButtonPressCallback(self,obj,event):
        if self.PickMode != 'network':
            return
        if self.vmtkRenderer.RenderWindowInteractor.GetControlKey() == 0:
            return
        if self.Network.GetNumberOfCells() == 0:
            return
        cellId = self.SelectedCellId
        if cellId == -1:
            return
        subId = self.SelectedSubId
        pcoords = self.SelectedPCoords
        cell = self.Network.GetCell(cellId)
        if pcoords < 0.5:
            pointId = cell.GetPointIds().GetId(subId)
        else:
            pointId = cell.GetPointIds().GetId(subId+1)
        point = self.Network.GetPoint(pointId)
        radius = self.NetworkRadiusArray.GetValue(pointId)
        if self.OperationMode == 'add':
            if radius == 0.0:
                radius = 1.0
            if self.ActiveSegmentSeeds.GetNumberOfPoints() == 0:
                self.ActiveSegmentSeedsPoints.InsertNextPoint(point)
                self.ActiveSegmentSeedsRadiusArray.InsertNextValue(radius)
                self.ActiveSegmentSeeds.Modified()
                self.AttachedCellIds[0] = cellId
                self.AttachedSubIds[0] = subId
                self.AttachedPCoords[0] = pcoords
                self.Render()
                return
            if self.AttachedCellIds[1] != -1:
                return
            self.AttachedCellIds[1] = cellId
            self.AttachedSubIds[1] = subId
            self.AttachedPCoords[1] = pcoords
            if self.ActiveSegmentPoints.GetNumberOfPoints() == 0:
                self.ActiveSegmentPoints.InsertNextPoint(self.ActiveSegmentSeedsPoints.GetPoint(0))
                self.ActiveSegmentRadiusArray.InsertNextValue(self.ActiveSegmentSeedsRadiusArray.GetValue(0))
            self.ActiveSegmentSeedsPoints.InsertNextPoint(point)
            self.ActiveSegmentSeedsRadiusArray.InsertNextValue(radius)
            self.ActiveSegmentSeeds.Modified()
            self.ActiveSegmentPoints.InsertNextPoint(point)
            self.ActiveSegmentRadiusArray.InsertNextValue(radius)
            self.ActiveSegmentCellArray.Initialize()
            numberOfPoints = self.ActiveSegmentPoints.GetNumberOfPoints()
            self.ActiveSegmentCellArray.InsertNextCell(numberOfPoints)
            for i in range(numberOfPoints):
                self.ActiveSegmentCellArray.InsertCellPoint(i)
            self.ActiveSegment.Modified()
            self.Render()
        elif self.OperationMode == 'delete':
            self.InitializeSelection()
            self.RemoveCell(cellId)
            self.NetworkTube.Modified()
            self.UpdateLabels()
        elif self.OperationMode == 'split':
            numberOfCellPoints = cell.GetNumberOfPoints()
            self.SplitCell(cellId,subId,pcoords)
            self.NetworkTube.Modified()
            self.UpdateLabels()
        elif self.OperationMode == 'label':
            self.LabelCellId = cellId
            label = self.NetworkLabelsArray.GetValue(cellId)
            if label:
                self.PrintLog("Current label: %s" % label)
            self.vmtkRenderer.PromptAsync("Please input new label: ",self.QueryLabelCallback)
        elif self.OperationMode == 'merge':
            if self.ActiveSegment.GetNumberOfCells() > 1:
                self.InitializeActiveSegment()
                self.CellIdsToMerge = []
            self.CellIdsToMerge.append(cellId)
            numberOfCellPoints = cell.GetNumberOfPoints()
            activePointIds = []
            for i in range(numberOfCellPoints):
                pointId = cell.GetPointId(i)
                point = self.Network.GetPoint(pointId)
                radius = self.NetworkRadiusArray.GetTuple1(pointId)
                activePointId = self.ActiveSegmentPoints.InsertNextPoint(point)
                self.ActiveSegmentRadiusArray.InsertNextValue(radius)
                activePointIds.append(activePointId)
            self.ActiveSegmentCellArray.InsertNextCell(numberOfCellPoints)
            for i in range(numberOfCellPoints):
                self.ActiveSegmentCellArray.InsertCellPoint(activePointIds[i])
            self.ActiveSegment.Modified()
            self.NetworkTube.Modified()
            self.Render()

    def QueryLabelCallback(self,newLabel):
        self.NetworkLabelsArray.SetValue(self.LabelCellId,newLabel)
        self.LabelCellId = None
        self.InitializeSelection()
        self.Network.Modified()
        self.UpdateLabels()

    def InitializeSelection(self):
        self.SelectionPoints.SetNumberOfPoints(0)
        self.SelectionRadiusArray.SetNumberOfValues(0)
        self.SelectedCellId = -1
        self.SelectedSubId = -1
        self.SelectedPCoords = 0.0
        self.Selection.Modified()
        self.Render()

    def InitializeActiveSegment(self):
        self.ActiveSegmentSeedsPoints.SetNumberOfPoints(0)
        self.ActiveSegmentSeedsRadiusArray.SetNumberOfValues(0)
        self.ActiveSegmentSeeds.Modified()
        self.ActiveSegmentPoints.SetNumberOfPoints(0)
        self.ActiveSegmentCellArray.Initialize()
        self.ActiveSegmentCellArray.SetNumberOfCells(0)
        self.ActiveSegmentRadiusArray.SetNumberOfValues(0)
        self.ActiveSegment.Modified()
        self.AttachedCellIds = [-1,-1]
        self.AttachedSubIds = [-1,-1]
        self.AttachedPCoords = [0.0,0.0]
        self.Render()

    def MouseMoveCallback(self,obj,event):
        if self.PickMode != 'network':
            return
        if self.vmtkRenderer.RenderWindowInteractor.GetControlKey() == 0:
            if self.SelectedCellId != -1:
                self.InitializeSelection()
            return
        if self.Network.GetNumberOfCells() == 0:
            return
        eventPosition = self.vmtkRenderer.RenderWindowInteractor.GetEventPosition()
        result = self.CellPicker.Pick(float(eventPosition[0]),float(eventPosition[1]),0.0,self.vmtkRenderer.Renderer)
        if result == 0:
            return
        cellId = self.CellPicker.GetCellId()
        cell = self.Network.GetCell(cellId)
        subId = self.CellPicker.GetSubId()
        pcoords = self.CellPicker.GetPCoords()[0]
        if pcoords < 0.5:
            pointId = cell.GetPointIds().GetId(subId)
        else:
            pointId = cell.GetPointIds().GetId(subId+1)
        if self.vmtkRenderer.RenderWindowInteractor.GetShiftKey() == 1:
            if subId < cell.GetNumberOfPoints()/2:
                pointId = cell.GetPointIds().GetId(0)
            else:
                pointId = cell.GetPointIds().GetId(cell.GetNumberOfPoints()-1)
        point = self.Network.GetPoint(pointId)
        radius = self.NetworkRadiusArray.GetValue(pointId)
        if radius == 0.0:
            radius = 1.0
        self.SelectionPoints.InsertPoint(0,point)
        self.SelectionRadiusArray.InsertValue(0,radius)
        self.SelectedCellId = cellId
        self.SelectedSubId = subId
        self.SelectedPCoords = pcoords
        self.Selection.Modified()
        self.Render()

    def PlaneStartInteractionCallback(self,obj,event):
        if self.PickMode != 'image':
            return
        if self.OperationMode != 'add':
            return
        if self.vmtkRenderer.RenderWindowInteractor.GetControlKey() == 0:
            return
        point = obj.GetCurrentCursorPosition()
        radius = self.CurrentRadius
        if self.ActiveSegmentSeeds.GetNumberOfPoints() == 0:
            self.ActiveSegmentSeedsPoints.InsertNextPoint(point)
            self.ActiveSegmentSeedsRadiusArray.InsertNextValue(radius)
            self.ActiveSegmentSeeds.Modified()
            self.Render()
            return
        if self.AttachedCellIds[1] != -1:
            return
        if self.ActiveSegmentPoints.GetNumberOfPoints() == 0:
            self.ActiveSegmentPoints.InsertNextPoint(self.ActiveSegmentSeedsPoints.GetPoint(0))
            self.ActiveSegmentRadiusArray.InsertNextValue(self.ActiveSegmentSeedsRadiusArray.GetValue(0))
        self.ActiveSegmentSeedsPoints.InsertNextPoint(point)
        self.ActiveSegmentSeedsRadiusArray.InsertNextValue(radius)
        self.ActiveSegmentSeeds.Modified()
        self.ActiveSegmentPoints.InsertNextPoint(point)
        self.ActiveSegmentRadiusArray.InsertNextValue(radius)
        self.ActiveSegmentCellArray.Initialize()
        numberOfPoints = self.ActiveSegmentPoints.GetNumberOfPoints()
        self.ActiveSegmentCellArray.InsertNextCell(numberOfPoints)
        for i in range(numberOfPoints):
            self.ActiveSegmentCellArray.InsertCellPoint(i)
        self.ActiveSegment.Modified()
        self.Render()

    def FirstRender(self):
        self.vmtkRenderer.Render()

    def Render(self):
        self.vmtkRenderer.RenderWindow.Render()

    def UpdateAndRender(self):
        self.vmtkRenderer.Render(interactive=0)

    def RunActiveTube(self,segment):
        activeTubes = vmtkactivetubes.vmtkActiveTubes()
        activeTubes.Centerline = segment
        activeTubes.Image = self.FeatureImage
        activeTubes.NumberOfIterations = self.NumberOfIterations
        activeTubes.PotentialWeight = self.PotentialWeight
        activeTubes.StiffnessWeight = self.StiffnessWeight
        activeTubes.RadiusArrayName = self.RadiusArrayName
        activeTubes.Execute()
        segment.DeepCopy(activeTubes.Centerline)

    def Execute(self):
 
        if not self.Network:
            self.Network = vtk.vtkPolyData()
            networkPoints = vtk.vtkPoints()
            networkLines = vtk.vtkCellArray()
            radiusArray = vtk.vtkDoubleArray()
            radiusArray.SetName(self.RadiusArrayName)
            self.Network.SetPoints(networkPoints)
            self.Network.SetLines(networkLines)
            self.Network.GetPointData().AddArray(radiusArray)
        
        if not self.vmtkRenderer:
            self.vmtkRenderer = vmtkrenderer.vmtkRenderer()
            self.vmtkRenderer.Initialize()
            self.OwnRenderer = 1

        self.vmtkRenderer.ExitAfterTextInputMode = False
        self.vmtkRenderer.RegisterScript(self) 

        if self.Image and (not self.PlaneWidgetX or not self.PlaneWidgetY or not self.PlaneWidgetZ):
            imageViewer = vmtkimageviewer.vmtkImageViewer()
            imageViewer.Image = self.Image
            imageViewer.vmtkRenderer = self.vmtkRenderer
            imageViewer.Display = 0
            imageViewer.Execute()
            self.PlaneWidgetX = imageViewer.PlaneWidgetX
            self.PlaneWidgetY = imageViewer.PlaneWidgetY
            self.PlaneWidgetZ = imageViewer.PlaneWidgetZ

        if self.Image:
            spacing = self.Image.GetSpacing()
            self.CurrentRadius = min(spacing)

        if self.UseActiveTubes and not self.FeatureImage:
            imageFeatures = vmtkimagefeatures.vmtkImageFeatures()
            imageFeatures.Image = self.Image
            imageFeatures.FeatureImageType = 'vtkgradient'
            imageFeatures.Execute()
            self.FeatureImage = imageFeatures.FeatureImage
 
        self.NetworkRadiusArray = self.Network.GetPointData().GetArray(self.RadiusArrayName)

        self.Network.GetPointData().SetActiveScalars(self.RadiusArrayName)

        networkMapper = vtk.vtkPolyDataMapper()
        networkMapper.SetInput(self.Network)
        networkMapper.SetScalarModeToUseCellData()

        self.NetworkActor = vtk.vtkActor()
        self.NetworkActor.SetMapper(networkMapper)
        self.vmtkRenderer.Renderer.AddActor(self.NetworkActor)

        self.NetworkTube = vtk.vtkTubeFilter()
        self.NetworkTube.SetInput(self.Network)
        self.NetworkTube.SetVaryRadiusToVaryRadiusByAbsoluteScalar()
        self.NetworkTube.SetNumberOfSides(20)
        networkTubeMapper = vtk.vtkPolyDataMapper()
        networkTubeMapper.SetInput(self.NetworkTube.GetOutput())
        networkTubeMapper.ScalarVisibilityOff()
        networkTubeActor = vtk.vtkActor()
        networkTubeActor.SetMapper(networkTubeMapper)
        networkTubeActor.PickableOff()
        networkTubeActor.GetProperty().SetOpacity(0.2)
        self.vmtkRenderer.Renderer.AddActor(networkTubeActor)

        self.Selection = vtk.vtkPolyData()
        self.SelectionPoints = vtk.vtkPoints()
        self.SelectionRadiusArray = vtk.vtkDoubleArray()
        self.SelectionRadiusArray.SetName(self.RadiusArrayName)
        self.Selection.SetPoints(self.SelectionPoints)
        self.Selection.GetPointData().AddArray(self.SelectionRadiusArray)
        self.Selection.GetPointData().SetActiveScalars(self.RadiusArrayName)

        glyphs = vtk.vtkGlyph3D()
        glyphSource = vtk.vtkSphereSource()
        glyphSource.SetRadius(1.0)
        glyphSource.SetThetaResolution(20)
        glyphSource.SetPhiResolution(20)
        glyphs.SetInput(self.Selection)
        glyphs.SetSource(glyphSource.GetOutput())
        glyphs.SetScaleModeToScaleByScalar()
        glyphs.SetScaleFactor(1.0)

        selectionMapper = vtk.vtkPolyDataMapper()
        selectionMapper.SetInput(glyphs.GetOutput())

        self.SelectionActor = vtk.vtkActor()
        self.SelectionActor.SetMapper(selectionMapper)
        self.SelectionActor.GetProperty().SetColor(1.0,0.0,0.0)
        self.SelectionActor.GetProperty().SetOpacity(0.5)
        self.SelectionActor.PickableOff()
        self.vmtkRenderer.Renderer.AddActor(self.SelectionActor)

        self.ActiveSegmentSeeds = vtk.vtkPolyData()
        self.ActiveSegmentSeedsPoints = vtk.vtkPoints()
        self.ActiveSegmentSeedsRadiusArray = vtk.vtkDoubleArray()
        self.ActiveSegmentSeedsRadiusArray.SetName(self.RadiusArrayName)
        self.ActiveSegmentSeeds.SetPoints(self.ActiveSegmentSeedsPoints)
        self.ActiveSegmentSeeds.GetPointData().AddArray(self.ActiveSegmentSeedsRadiusArray)
        self.ActiveSegmentSeeds.GetPointData().SetActiveScalars(self.RadiusArrayName)

        activeSegmentSeedsGlyphs = vtk.vtkGlyph3D()
        activeSegmentSeedsGlyphSource = vtk.vtkSphereSource()
        activeSegmentSeedsGlyphSource.SetRadius(1.0)
        activeSegmentSeedsGlyphSource.SetThetaResolution(20)
        activeSegmentSeedsGlyphSource.SetPhiResolution(20)
        activeSegmentSeedsGlyphs.SetInput(self.ActiveSegmentSeeds)
        activeSegmentSeedsGlyphs.SetSource(activeSegmentSeedsGlyphSource.GetOutput())
        activeSegmentSeedsGlyphs.SetScaleModeToScaleByScalar()
        activeSegmentSeedsGlyphs.SetScaleFactor(1.0)

        activeSegmentSeedsMapper = vtk.vtkPolyDataMapper()
        activeSegmentSeedsMapper.SetInput(activeSegmentSeedsGlyphs.GetOutput())
        activeSegmentSeedsMapper.ScalarVisibilityOff()

        self.ActiveSegmentSeedsActor = vtk.vtkActor()
        self.ActiveSegmentSeedsActor.SetMapper(activeSegmentSeedsMapper)
        self.ActiveSegmentSeedsActor.GetProperty().SetColor(1.0,0.0,0.0)
        self.ActiveSegmentSeedsActor.GetProperty().SetOpacity(0.5)
        self.ActiveSegmentSeedsActor.PickableOff()
        self.vmtkRenderer.Renderer.AddActor(self.ActiveSegmentSeedsActor)

        self.ActiveSegment = vtk.vtkPolyData()
        self.ActiveSegmentPoints = vtk.vtkPoints()
        self.ActiveSegmentCellArray = vtk.vtkCellArray()
        self.ActiveSegmentRadiusArray = vtk.vtkDoubleArray()
        self.ActiveSegmentRadiusArray.SetName(self.RadiusArrayName)
        self.ActiveSegment.SetPoints(self.ActiveSegmentPoints)
        self.ActiveSegment.SetLines(self.ActiveSegmentCellArray)
        self.ActiveSegment.GetPointData().AddArray(self.ActiveSegmentRadiusArray)
        self.ActiveSegment.GetPointData().SetActiveScalars(self.RadiusArrayName)

        activeSegmentMapper = vtk.vtkPolyDataMapper()
        activeSegmentMapper.ScalarVisibilityOff()
        if self.SplineInterpolation and self.Image != None:
            splineFilter = vtk.vtkSplineFilter()
            splineFilter.SetInput(self.ActiveSegment)
            splineFilter.SetSubdivideToLength()
            splineFilter.SetLength(2.0*min(self.Image.GetSpacing()))
            activeSegmentMapper.SetInput(splineFilter.GetOutput())
        else:
            activeSegmentMapper.SetInput(self.ActiveSegment)

        self.ActiveSegmentActor = vtk.vtkActor()
        self.ActiveSegmentActor.SetMapper(activeSegmentMapper)
        self.ActiveSegmentActor.GetProperty().SetColor(1.0,1.0,1.0)
        self.ActiveSegmentActor.GetProperty().SetLineWidth(3.0)
        self.ActiveSegmentActor.PickableOff()
        self.vmtkRenderer.Renderer.AddActor(self.ActiveSegmentActor)

        activeTube = vtk.vtkTubeFilter()
        activeTube.SetInput(activeSegmentMapper.GetInput())
        activeTube.SetVaryRadiusToVaryRadiusByAbsoluteScalar()
        activeTube.SetNumberOfSides(20)
        activeTubeMapper = vtk.vtkPolyDataMapper()
        activeTubeMapper.SetInput(activeTube.GetOutput())
        activeTubeMapper.ScalarVisibilityOff()
        activeTubeActor = vtk.vtkActor()
        activeTubeActor.SetMapper(activeTubeMapper)
        activeTubeActor.PickableOff()
        activeTubeActor.GetProperty().SetOpacity(0.6)
        self.vmtkRenderer.Renderer.AddActor(activeTubeActor)

        self.NetworkLabelsArray = vtk.vtkStringArray.SafeDownCast(self.Network.GetCellData().GetAbstractArray(self.LabelsArrayName))
        if not self.NetworkLabelsArray:
            self.NetworkLabelsArray = vtk.vtkStringArray()
            self.NetworkLabelsArray.SetName(self.LabelsArrayName)
            self.NetworkLabelsArray.SetNumberOfValues(self.Network.GetNumberOfCells())
            for i in range(self.Network.GetNumberOfCells()):
                self.NetworkLabelsArray.SetValue(i,'')
            self.Network.GetCellData().AddArray(self.NetworkLabelsArray)
        self.CellCenters = vtk.vtkCellCenters()
        self.CellCenters.SetInput(self.Network)
        self.CellCenters.VertexCellsOff()
        self.CellCenters.Update()

        labeledMapper = vtk.vtkLabeledDataMapper()
        labeledMapper.SetInput(self.CellCenters.GetOutput())
        labeledMapper.SetLabelModeToLabelFieldData()
        labeledMapper.SetFieldDataName(self.LabelsArrayName)
        labeledMapper.GetLabelTextProperty().SetFontFamilyToArial()
        labeledMapper.GetLabelTextProperty().BoldOff()
        labeledMapper.GetLabelTextProperty().ItalicOff()
        labeledMapper.GetLabelTextProperty().ShadowOff()
        
        self.LabelsActor = vtk.vtkActor2D()
        self.LabelsActor.SetMapper(labeledMapper)
        self.LabelsActor.VisibilityOff()
        self.vmtkRenderer.Renderer.AddActor(self.LabelsActor)

        self.CellPicker = vtk.vtkCellPicker()
        self.CellPicker.SetTolerance(1E-2)
        self.CellPicker.InitializePickList()
        self.CellPicker.AddPickList(self.NetworkActor)
        self.CellPicker.PickFromListOn()

        self.vmtkRenderer.AddKeyBinding('a','Add mode.',self.AddCallback)
        self.vmtkRenderer.AddKeyBinding('d','Delete mode.',self.DeleteCallback)
        self.vmtkRenderer.AddKeyBinding('m','Merge mode.',self.MergeCallback)
        self.vmtkRenderer.AddKeyBinding('s','Split mode.',self.SplitCallback)
        self.vmtkRenderer.AddKeyBinding('l','Label mode.',self.LabelCallback)
        self.vmtkRenderer.AddKeyBinding('Tab','Show labels.',self.ShowLabelCallback)
        self.vmtkRenderer.RenderWindowInteractor.AddObserver("KeyReleaseEvent", self.KeyReleaseCallback)
        self.vmtkRenderer.RenderWindowInteractor.AddObserver("LeftButtonPressEvent", self.LeftButtonPressCallback)
        self.vmtkRenderer.RenderWindowInteractor.AddObserver("MouseMoveEvent", self.MouseMoveCallback)

        if self.PlaneWidgetX:
            self.PlaneWidgetX.UseContinuousCursorOn()
            self.PlaneWidgetX.AddObserver("StartInteractionEvent", self.PlaneStartInteractionCallback)
        if self.PlaneWidgetY:
            self.PlaneWidgetY.UseContinuousCursorOn()
            self.PlaneWidgetY.AddObserver("StartInteractionEvent", self.PlaneStartInteractionCallback)
        if self.PlaneWidgetZ:
            self.PlaneWidgetZ.UseContinuousCursorOn()
            self.PlaneWidgetZ.AddObserver("StartInteractionEvent", self.PlaneStartInteractionCallback)

        self.FirstRender()

        self.Surface = self.NetworkTube.GetOutput()

        if self.OwnRenderer:
            self.vmtkRenderer.Deallocate()

      
if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()




#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkmeshlambda2.py,v $
## Language:  Python
## Date:      $Date: 2005/09/14 09:49:59 $
## Version:   $Revision: 1.6 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENCE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even 
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
##      PURPOSE.  See the above copyright notices for more information.

import vtkvmtk
import vtk
import sys

import pypes

vmtkmeshlambda2 = 'vmtkMeshLambda2'

class vmtkMeshLambda2(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Mesh = None

        self.VelocityArrayName = None
        self.Lambda2ArrayName = 'Lambda2'

        self.ConvergenceTolerance = 1E-6
        self.QuadratureOrder = 3

        self.SetScriptName('vmtkmeshlambda2')
        self.SetScriptDoc('compute lambda2 from a velocity field')
        self.SetInputMembers([
            ['Mesh','i','vtkUnstructuredGrid',1,'','the input mesh','vmtkmeshreader'],
            ['VelocityArrayName','velocityarray','str',1,'',''],
            ['Lambda2ArrayName','lambda2array','str',1,'',''],
            ['ConvergenceTolerance','tolerance','float',1,'',''],
            ['QuadratureOrder','quadratureorder','int',1,'','']
            ])
        self.SetOutputMembers([
            ['Mesh','o','vtkUnstructuredGrid',1,'','the output mesh','vmtkmeshwriter']
            ])

    def Execute(self):

        if (self.Mesh == None):
            self.PrintError('Error: no Mesh.')

        lambda2Filter = vtkvmtk.vtkvmtkMeshLambda2()
        lambda2Filter.SetInput(self.Mesh)
        lambda2Filter.SetVelocityArrayName(self.VelocityArrayName)
        lambda2Filter.SetLambda2ArrayName(self.Lambda2ArrayName)
        lambda2Filter.SetConvergenceTolerance(self.ConvergenceTolerance)
        lambda2Filter.SetQuadratureOrder(self.QuadratureOrder)
        lambda2Filter.ComputeIndividualPartialDerivativesOn()
        lambda2Filter.ForceBoundaryToNegativeOn()
        lambda2Filter.Update()

        self.Mesh = lambda2Filter.GetOutput()

        if self.Mesh.GetSource():
            self.Mesh.GetSource().UnRegisterAllOutputs()


if __name__=='__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtksurfaceappend.py,v $
## Language:  Python
## Date:      $Date: 2005/09/14 09:49:59 $
## Version:   $Revision: 1.4 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENCE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even 
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
##      PURPOSE.  See the above copyright notices for more information.

import vtk
import sys

import pypes

vmtksurfaceappend = 'vmtkSurfaceAppend'

class vmtkSurfaceAppend(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Surface = None
        self.Surface2 = None

        self.SetScriptName('vmtksurfaceappend')
        self.SetScriptDoc('scale a surface by an isotropic factor')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','the input surface','vmtksurfacereader'],
            ['Surface2','i2','vtkPolyData',1,'','the second input surface','vmtksurfacereader'],
            ])
        self.SetOutputMembers([
            ['Surface','o','vtkPolyData',1,'','the output surface','vmtksurfacewriter']
            ])

    def Execute(self):

        if (self.Surface == None):
            self.PrintError('Error: no Surface.')

        appendFilter = vtk.vtkAppendPolyData()
        appendFilter.AddInput(self.Surface)
        appendFilter.AddInput(self.Surface2)
        appendFilter.Update()

        self.Surface = appendFilter.GetOutput()

        if self.Surface.GetSource():
            self.Surface.GetSource().UnRegisterAllOutputs()


if __name__=='__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()
